# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt


from five import grok
from AccessControl.security import checkPermission

from silva.core.interfaces import IVersionedObject
from silva.core.views import views as silvaviews
from silva.core.smi.content.publish import Publish
from zeam.form import silva as silvaforms


class PublicationInfo(silvaviews.Viewlet):
    """Portlet giving information about the publication status.
    """
    grok.context(IVersionedObject)
    grok.view(Publish)
    grok.viewletmanager(silvaforms.SMIFormPortlets)
    grok.order(10)

    def update(self):
        format = self.request.locale.dates.getFormatter('dateTime', 'short').format
        convert = lambda d: d is not None and format(d.asdatetime()) or None
        self.first_publication_date = convert(
            self.context.get_first_publication_date())
        self.publication_date = convert(
            self.context.get_public_version_publication_datetime())
        self.expiration_date = convert(
            self.context.get_public_version_expiration_datetime())
        self.have_unapproved = self.context.get_unapproved_version() != None
        self.have_next = self.context.get_next_version() != None
        self.have_closed = self.context.get_last_closed_version() != None
        self.have_approved = self.context.is_approved()
        self.have_published = self.context.is_published()
        self.may_approve = checkPermission('silva.ApproveSilvaContent', self.context)
        self.may_change = checkPermission('silva.ChangeSilvaContent', self.context)




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from silva.core.interfaces import IAutoTOC
from zope.interface.verify import verifyObject

from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings


class ReaderAutoTOCTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addAutoTOC('contents', 'Table of Content')

    def test_autotoc_access(self):
        """A reader cannot create an autotoc, however he look at it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('reader')
        self.assertEqual(browser.inspect.title, u'root')

        # We should see our auto toc.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Table of Content',
              'identifier': 'contents',
              'author': 'editor'}])
        # Select it.
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(browser.inspect.toolbar, ['Copy'])
        self.assertEqual(browser.inspect.listing[0].goto_dropdown.click(), 200)
        self.assertEqual(browser.inspect.listing[0].goto_actions, ['Preview'])

        # Go on it. We arrive on the edit tab, where we cannot do a thing.
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, u'Table of Content')
        self.assertEqual(browser.inspect.toolbar, [])
        self.assertEqual(browser.inspect.tabs, ['Edit', 'Preview'])
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        self.assertEqual(browser.inspect.views, ['View...'])
        self.assertEqual(browser.inspect.form, ['Edit a Silva AutoTOC'])
        edit_form = browser.inspect.form['Edit a Silva AutoTOC']
        self.assertEqual(edit_form.fields, [])
        self.assertEqual(edit_form.actions, ['Back'])

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])


class AuthorAutoTOCTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'
    access = False

    def setUp(self):
        self.root = self.layer.get_application()

    def test_autotoc_add_and_listing(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva AutoTOC', browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva AutoTOC'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva AutoTOC"])
        self.assertEqual(browser.inspect.toolbar, [])

        add_form = browser.inspect.form['Add a Silva AutoTOC']
        add_form.fields['id'].value = 'contents'
        add_form.fields['title'].value = u'Table des matières'
        self.assertEqual(add_form.actions, ['Cancel', 'Save'])
        self.assertEqual(add_form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva AutoTOC.")
        # We should have now an AutoTOC with the id 'contents'
        self.assertTrue(verifyObject(IAutoTOC, self.root._getOb('contents', None)))

        # Inspect newly created content.
        self.assertEqual(browser.inspect.title, u'Table des matières')
        self.assertEqual(
            browser.inspect.tabs,
            ['Edit', 'Preview', 'Properties', 'Settings'])
        self.assertEqual(
            browser.inspect.views,
            ['View...'])
        # We are on contents
        self.assertEqual(
            browser.inspect.activetabs,
            ['Edit'])

        self.assertEqual(browser.inspect.form, ['Edit a Silva AutoTOC'])
        edit_form = browser.inspect.form['Edit a Silva AutoTOC']
        self.assertNotEqual(edit_form.fields, [])
        self.assertEqual(edit_form.actions, ['Back', 'Save changes'])
        self.assertIn('Depth', edit_form.fields)
        edit_form.fields['Depth'].value = '10'
        self.assertEqual(edit_form.actions['Save changes'].click(), 200)
        browser.macros.assertFeedback('Changes saved.')

        # No special actions in toolbar
        self.assertEqual(browser.inspect.toolbar, [])

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our auto toc
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Table des matières',
              'identifier': 'contents',
              'author': self.user}])
        # Select it
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename'])
        self.assertEqual(
            browser.inspect.listing[0].goto_dropdown.click(),
            200)
        self.assertEqual(
            browser.inspect.listing[0].goto_actions,
            ['Preview', 'Properties'])

        # Delete the autotoc
        self.assertEqual(browser.inspect.toolbar['Delete'].click(), 200)
        # Content is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Table des matières', 'identifier': 'contents'}])
        self.assertEqual(
            browser.inspect.dialog,
            [{'title': 'Confirm deletion'}])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(browser.inspect.listing, [])
        browser.macros.assertFeedback(u'Deleted "Table des matières".')
        self.assertIs(self.root._getOb('contents', None), None)


class EditorAutoTOCTestCase(AuthorAutoTOCTestCase):
    user = 'editor'


class ChiefEditorAutoTOCTestCase(EditorAutoTOCTestCase):
    user = 'chiefeditor'


class ManagerAutoTOCTestCase(ChiefEditorAutoTOCTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReaderAutoTOCTestCase))
    suite.addTest(unittest.makeSuite(AuthorAutoTOCTestCase))
    suite.addTest(unittest.makeSuite(EditorAutoTOCTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorAutoTOCTestCase))
    suite.addTest(unittest.makeSuite(ManagerAutoTOCTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings
from silva.core.references.reference import get_content_id


class EditorGhostFolderTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'editor'
    goto = []

    def setUp(self):
        """Set up objects for test :

           + root (Silva root)
               |
               + folder (Silva Folder)
                   |
                   +- document (Silva Mockup)
                   |
                   +- publication (Silva Publication)
                       |
                       +- document (Silva Mockup)
        """
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Folder')
            folder = self.root._getOb('folder')

            factory = folder.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')
            factory.manage_addPublication('publication', 'Publication')
            publication = folder._getOb('publication')

            factory = publication.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')

    def test_ghost_folder_add_and_listing(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva Ghost Folder', browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva Ghost Folder'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva Ghost Folder"])
        self.assertEqual(browser.inspect.toolbar, [])

        form = browser.inspect.form['Add a Silva Ghost Folder']
        self.assertIn('Id', form.fields)
        form.fields['Id'].value = 'ghost_folder'
        # Lookup an element to ghost
        self.assertEqual(browser.inspect.reference[0].click(), 200)
        self.assertEqual(browser.inspect.dialog, ['Lookup an item'])
        dialog = browser.inspect.dialog['Lookup an item']
        self.assertEqual(dialog.buttons, ['Cancel'])
        self.assertEqual(dialog.listing, ['current container', 'folder'])
        self.assertEqual(dialog.listing['folder'].title, 'Folder')
        self.assertEqual(dialog.listing['folder'].select.click(), 200)
        # This should have closed the dialog and selected the document
        # in the hidden field
        self.assertEqual(browser.inspect.dialog, [])

        self.assertEqual(form.actions, ['Cancel', 'Save'])
        self.assertEqual(form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva Ghost Folder.")
        # Inspect newly created content

        self.assertEqual(browser.inspect.title, u'Folder')
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Edit', 'Preview', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Content'])
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': self.user},
             {'title': u'Publication',
              'identifier': 'publication',
              'author': self.user}])

        # Go though the tabs
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Check edit form
        self.assertEqual(browser.inspect.form, ['Edit a Silva Ghost Folder'])
        form = browser.inspect.form['Edit a Silva Ghost Folder']
        self.assertNotEqual(form.fields, [])
        self.assertIn('Haunted', form.fields)
        self.assertEqual(
            form.fields['Haunted'].value,
            str(get_content_id(self.root.folder)))
        self.assertEqual(form.actions, ['Back', 'Save changes', 'Synchronize'])
        self.assertEqual(form.actions['Synchronize'].click(), 200)
        browser.macros.assertFeedback(u"Ghost Folder synchronized.")

        self.assertEqual(browser.inspect.tabs['Content'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our ghost and the original document.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Folder',
              'identifier': 'folder',
              'author': u'editor'},
             {'title': u'Folder',
              'identifier': 'ghost_folder',
              'author': self.user}])
        # Select it
        self.assertEqual(browser.inspect.listing[1].identifier.click(), 200)
        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename', 'Publish'])
        self.assertEqual(
            browser.inspect.listing[1].goto_dropdown.click(),
            200)
        self.assertEqual(
            browser.inspect.listing[1].goto_actions,
            ['Preview', 'Properties'] + self.goto)

        self.assertEqual(browser.inspect.toolbar['Delete'].click(), 200)
        # Content is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Folder',
              'identifier': 'folder',
              'author': u'editor'},
             {'title': u'Folder',
              'identifier': 'ghost_folder',
              'author': self.user}])
        self.assertEqual(browser.inspect.dialog, ['Confirm deletion'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Folder',
              'identifier': 'folder',
              'author': u'editor'}])
        browser.macros.assertFeedback(u'Deleted "Folder".')


class ChiefEditorGhostFolderTestCase(EditorGhostFolderTestCase):
    user = 'chiefeditor'
    goto = ['Access']


class ManagerGhostFolderTestCase(ChiefEditorGhostFolderTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(EditorGhostFolderTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorGhostFolderTestCase))
    suite.addTest(unittest.makeSuite(ManagerGhostFolderTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings
from silva.core.references.reference import get_content_id


class AuthorGhostTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'
    toolbar = ['Request approval']

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')

    def test_ghost_add_and_listing(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva Ghost', browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva Ghost'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva Ghost"])
        self.assertEqual(browser.inspect.toolbar, [])

        form = browser.inspect.form['Add a Silva Ghost']
        self.assertIn('Id', form.fields)
        form.fields['Id'].value = 'ghost_document'
        # Lookup an element to ghost
        self.assertEqual(browser.inspect.reference[0].click(), 200)
        self.assertEqual(browser.inspect.dialog, ['Lookup an item'])
        dialog = browser.inspect.dialog['Lookup an item']
        self.assertEqual(dialog.buttons, ['Cancel'])
        self.assertEqual(dialog.listing, ['current container', 'document'])
        self.assertEqual(dialog.listing['document'].title, 'Document')
        self.assertEqual(dialog.listing['document'].select.click(), 200)
        # This should have closed the dialog and selected the document
        # in the hidden field
        self.assertEqual(browser.inspect.dialog, [])
        self.assertIn('Haunted', form.fields)
        self.assertEqual(
            form.fields['Haunted'].value,
            str(get_content_id(self.root.document)))
        self.assertEqual(form.actions, ['Cancel', 'Save'])
        self.assertEqual(form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva Ghost.")

        # Inspect newly created content
        self.assertEqual(browser.inspect.title, u'Document')
        self.assertEqual(
            browser.inspect.tabs,
            ['Edit', 'Preview', 'Properties', 'Publish', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        self.assertEqual(browser.inspect.form, ['Edit a Silva Ghost'])
        form = browser.inspect.form['Edit a Silva Ghost']
        self.assertNotEqual(form.fields, [])
        self.assertIn('Haunted', form.fields)
        self.assertEqual(
            form.fields['Haunted'].value,
            str(get_content_id(self.root.document)))
        self.assertEqual(form.actions, ['Back', 'Save changes'])

        # An action let you directly request approval
        self.assertEqual(browser.inspect.toolbar, self.toolbar)

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our ghost and the original document.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': u'editor'},
             {'title': u'Document',
              'identifier': 'ghost_document',
              'author': self.user}])
        # Select it
        self.assertEqual(browser.inspect.listing[1].identifier.click(), 200)

        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename'] + self.toolbar)
        self.assertEqual(
            browser.inspect.listing[1].goto_dropdown.click(),
            200)
        self.assertEqual(
            browser.inspect.listing[1].goto_actions,
            ['Preview', 'Properties', 'Publish'])

        self.assertEqual(browser.inspect.toolbar['Delete'].click(), 200)
        # Content is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': u'editor'},
             {'title': u'Document',
              'identifier': 'ghost_document',
              'author': self.user}])
        self.assertEqual(browser.inspect.dialog, ['Confirm deletion'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': u'editor'}])
        browser.macros.assertFeedback(u'Deleted "Document".')


class EditorGhostTestCase(AuthorGhostTestCase):
    user = 'editor'
    toolbar = ['Publish']


class ChiefEditorGhostTestCase(EditorGhostTestCase):
    user = 'chiefeditor'


class ManagerGhostTestCase(ChiefEditorGhostTestCase):
    user = 'manager'




def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AuthorGhostTestCase))
    suite.addTest(unittest.makeSuite(EditorGhostTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorGhostTestCase))
    suite.addTest(unittest.makeSuite(ManagerGhostTestCase))
    return suite




# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok

from infrae.comethods import cofunction

from silva.core.interfaces import IContainer
from silva.core.interfaces import IPublicationWorkflow
from silva.core.interfaces.errors import VersioningError
from silva.core.smi.content.publish import IPublicationFields
from silva.translations import translate as _
from silva.ui.rest.container import get_container_changes
from silva.ui.rest.container.generator import ContentGenerator
from silva.ui.rest.container.notifier import ContentNotifier

from zeam.form import autofields
from zeam.form.silva.interfaces import IDefaultAction
from zeam.form.silva.interfaces import IRESTCloseOnSuccessAction
from zeam.form.silva.interfaces import IRESTExtraPayloadProvider
from zeam.form.ztk.actions import EditAction
from zeam.form import silva as silvaforms


class MultiApproveAction(EditAction):
    grok.implements(
        IDefaultAction,
        IRESTCloseOnSuccessAction,
        IRESTExtraPayloadProvider)
    title = _(u"Approve")

    def get_extra_payload(self, form):
        form.need(
            lambda form, data:
                get_container_changes(form, data['content']['extra']))
        return {'content': {
                'ifaces': ['listing-changes'],
                'actions': {}
                }}

    @cofunction
    def approver(self, form, data):
        content = yield
        while content is not None:
            result = None
            workflow = IPublicationWorkflow(content, None)
            if workflow is None:
                result = VersioningError(
                    _(u"This action is not applicable on this content."),
                    content)
            else:
                try:
                    self.applyData(form, form.dataManager(content), data)
                    IPublicationWorkflow(content).approve()
                except VersioningError as error:
                    result = error
                else:
                    result = content
            content = yield result

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            return silvaforms.FAILURE

        get_notifier = ContentNotifier(form.request)
        with self.approver(form, data) as approver:
            with get_notifier(
                approver,
                u"Approved for the future ${contents}.",
                u"Could not approve ${contents}: ${reason}") as notifier:
                with ContentGenerator(get_notifier.notify) as get_contents:
                    notifier.map(get_contents(
                            form.request.form.get('form.prefix.contents')))
        return silvaforms.SUCCESS



class ApproveForFuturePopupForm(silvaforms.RESTPopupForm):
    grok.context(IContainer)
    grok.name('silva.core.smi.approveforfuture')

    fields = autofields.FieldsCollector(IPublicationFields)
    dataManager = autofields.FieldsDataManager()
    actions = silvaforms.Actions(silvaforms.CancelAction(),
                                 MultiApproveAction(identifier='approve'))

    label = _('Approve item for future')
    description = _(u'Approve selected item for the future')
    ignoreContent = True
    ignoreRequest = False



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl import getSecurityManager
from five import grok
from zope.component import getUtility

from silva.ui.rest import UIREST
from silva.core.interfaces import ISilvaObject
from silva.core.references.interfaces import IReferenceService
from silva.core.references.reference import BrokenReferenceError
from silva.translations import translate as _
from zope.traversing.browser import absoluteURL
from zeam.form import silva as silvaforms
from zeam.form.silva.interfaces import IDefaultAction


class BreakReferencePermission(grok.Permission):
    grok.name('silva.breakreference')
    grok.title('Break silva references')


class BreakReferencesForm(silvaforms.PopupForm):
    """ Form for breaking references
    """
    grok.context(BrokenReferenceError)
    grok.template('break_references')
    grok.name('error.html')

    label = _(u"Broken references")
    actions = silvaforms.Actions(
        silvaforms.CancelAction())

    def get_info(self, content):
        return {
            'title': content.get_title(),
            'path': self.get_content_path(content),
            'url': absoluteURL(content, self.request)}

    def update(self):
        self.content = self.context.error.args[0].target
        self.info = self.get_info(self.content)

    def referrers(self):
        service = getUtility(IReferenceService)
        return (self.get_info(reference.source)
                for reference in service.get_references_to(self.content))

    def allowed_to_break(self):
        return getSecurityManager().checkPermission(
            'Break silva references', self.context)

    def updateForm(self):
        # XXX Hack properties to get URL working
        self.__parent__ = self.context
        self.__name__ = 'error.html'
        # Force form url to our content form.
        result = super(BreakReferencesForm, self).updateForm()
        result['content']['submit_url'] = '/'.join(
            (absoluteURL(self.content, self.request),
             '++rest++silva.core.smi.breakreferences'))
        self.response.setStatus(400)
        return result

    @silvaforms.action(
        _(u'Break references'),
        available=allowed_to_break,
        implements=IDefaultAction)
    def break_references(self):
        # This is a fake action to create the action.
        pass


class BreakReferenceContentForm(UIREST):
    grok.context(ISilvaObject)
    grok.name('silva.core.smi.breakreferences')
    grok.require('silva.breakreference')

    def POST(self):
        service = getUtility(IReferenceService)
        for reference in list(service.get_references_to(self.context)):
            reference.set_target_id(0)
        return self.json_response({
                'content': {
                    'ifaces': ['message'],
                    'title': self.translate(_(u"References broken")),
                    'message': self.translate(
                        _(u"References to ${content} (${path}) have been broken.",
                          mapping={'content': self.context.get_title_or_id(),
                                   'path': self.get_content_path(self.context)}))
                    }})



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok

from Products.Silva.mangle import Bytes
from Products.Silva.Publication import OverQuotaException
from silva.ui.rest.errors import ErrorREST
from silva.translations import translate as _


class OverQuotaError(ErrorREST):
    """ Over quota error.
    """
    grok.context(OverQuotaException)

    title = _(u'Over Quota')

    def update(self):
        self.exceeding_size = Bytes(self.context.error.args[0])



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt


import unittest

from silva.core.interfaces import ILink
from zope.interface.verify import verifyObject

from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings


class AuthorLinkTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'
    toolbar = ['Request approval']

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_link_add_internal_and_listing(self):
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva Link', browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva Link'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva Link"])
        self.assertEqual(browser.inspect.toolbar, [])

        # Add a new internal link
        form = browser.inspect.form['Add a Silva Link']
        self.assertIn('Id', form.fields)
        form.fields['Id'].value = 'silvacms'
        self.assertIn('Title', form.fields)
        form.fields['Title'].value = 'silvacms.org site'
        self.assertIn('Relative link', form.fields)
        form.fields['Relative link'].checked = True
        # Select an item to refer to.
        self.assertEqual(len(browser.inspect.reference), 1)
        self.assertEqual(browser.inspect.reference[0].click(), 200)
        self.assertEqual(browser.inspect.dialog, ['Lookup an item'])
        dialog = browser.inspect.dialog['Lookup an item']
        self.assertEqual(dialog.title, 'Lookup an item')
        self.assertEqual(dialog.buttons, ['Cancel', 'Clear'])
        self.assertEqual(dialog.listing, ['current container', 'document'])
        self.assertEqual(dialog.listing['document'].title, 'Document')
        self.assertEqual(dialog.listing['document'].select.click(), 200)
        # This select the item
        self.assertEqual(browser.inspect.dialog, [])
        self.assertEqual(form.actions, ['Cancel', 'Save'])
        self.assertEqual(form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva Link.")
        # We should have a link with the id 'silvacms'
        self.assertTrue(verifyObject(ILink, self.root._getOb('silvacms', None)))
        version = self.root.silvacms.get_editable()
        self.assertIsNot(version, None)
        self.assertEqual(version.get_relative(), True)
        self.assertEqual(version.get_target(), self.root.document)
        self.assertEqual(version.get_url(), '')

        # Inspect newly created content
        self.assertEqual(browser.inspect.title, u'silvacms.org site')
        # An action let you directly request approval
        self.assertEqual(browser.inspect.toolbar, self.toolbar)

    def test_link_add_external_and_listing(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva Link', browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva Link'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva Link"])
        self.assertEqual(browser.inspect.toolbar, [])

        form = browser.inspect.form['Add a Silva Link']
        self.assertIn('Id', form.fields)
        form.fields['Id'].value = 'silvacms'
        self.assertIn('Title', form.fields)
        form.fields['Title'].value = 'silvacms.org site'
        self.assertIn('URL', form.fields)
        form.fields['URL'].value = 'http://silvacms.org'
        self.assertIn('Relative link', form.fields)
        self.assertEqual(form.fields['Relative link'].checked, False)
        self.assertEqual(form.actions, ['Cancel', 'Save'])
        self.assertEqual(form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva Link.")
        self.assertTrue(verifyObject(ILink, self.root._getOb('silvacms', None)))
        version = self.root.silvacms.get_editable()
        self.assertIsNot(version, None)
        self.assertEqual(version.get_relative(), False)
        self.assertEqual(version.get_target(), None)
        self.assertEqual(version.get_url(), 'http://silvacms.org')

        # Inspect newly created content
        self.assertEqual(browser.inspect.title, u'silvacms.org site')
        self.assertEqual(
            browser.inspect.tabs,
            ['Edit', 'Preview', 'Properties', 'Publish', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        self.assertEqual(browser.inspect.form, ['Edit a Silva Link'])
        form = browser.inspect.form['Edit a Silva Link']
        self.assertNotEqual(form.fields, [])
        self.assertEqual(form.fields['Title'].value, 'silvacms.org site')
        self.assertEqual(form.fields['URL'].value, 'http://silvacms.org')
        self.assertEqual(form.fields['Relative link'].checked, False)
        self.assertEqual(form.actions, ['Back', 'Save changes'])

        # An action let you directly request approval
        self.assertEqual(browser.inspect.toolbar, self.toolbar)

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our ghost and the original document.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'silvacms.org site',
              'identifier': 'silvacms',
              'author': self.user}])
        # Select it
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)

        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename'] + self.toolbar)
        self.assertEqual(
            browser.inspect.listing[0].goto_dropdown.click(),
            200)
        self.assertEqual(
            browser.inspect.listing[0].goto_actions,
            ['Preview', 'Properties', 'Publish'])

        self.assertEqual(browser.inspect.toolbar['Delete'].click(), 200)
        # Content is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'silvacms.org site',
              'identifier': 'silvacms',
              'author': self.user}])
        self.assertEqual(browser.inspect.dialog, ['Confirm deletion'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(browser.inspect.listing, [])
        browser.macros.assertFeedback(u'Deleted "silvacms.org site".')
        self.assertIs(self.root._getOb('silvacms', None), None)


class EditorLinkTestCase(AuthorLinkTestCase):
    user = 'editor'
    toolbar = ['Publish']


class ChiefEditorLinkTestCase(EditorLinkTestCase):
    user = 'chiefeditor'


class ManagerLinkTestCase(ChiefEditorLinkTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AuthorLinkTestCase))
    suite.addTest(unittest.makeSuite(EditorLinkTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorLinkTestCase))
    suite.addTest(unittest.makeSuite(ManagerLinkTestCase))
    return suite




# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import zipfile
import uuid

from five import grok
from zope import schema
from zope.event import notify
from zope.interface import Interface
from zope.component import getUtility
from zope.lifecycleevent import ObjectModifiedEvent

from silva.core.references.interfaces import IReferenceService
from silva.core.conf import schema as silvaschema
from silva.core.interfaces import IContainer, IVersion, IGhostAware
from silva.core.interfaces import IZipFileImporter, IArchiveFileImporter
from silva.core.smi.content.container import Container, ContainerMenu
from silva.translations import translate as _
from silva.ui.menu import MenuItem
from zeam.form import silva as silvaforms


class IImportFields(Interface):
    """Import form description.
    """
    archive = silvaschema.Bytes(
        title=_("File to upload"),
        description=_(u"Locate a zip file containing assets "
                      u"or a zip file created by a Silva full media export."),
        required=True)
    asset_title = schema.TextLine(
        title=_(u"Title for assets"),
        description=_(u"Set a title for all assets created from the items "
                      u"of a zip file with assets."),
        default=u'',
        required=False)
    sub_directories = schema.Bool(
        title=_(u"Subdirectories"),
        description=_(u"Recreate the subdirectory structure of a zip file "
                      u"with assets by adding folders."),
        default=False,
        required=False)
    ignore_top_level = schema.Bool(
        title=_(u"Ignore top level container"),
        description=_(u"Do not recreate the top level container while "
                      u"importing a file. This let you import items that "
                      u"where exported from a container directly in the "
                      u"current one."))
    replace = schema.Bool(
        title=_(u"Replace items"),
        description=_(u"Replace items with the ones from the import file "
                      u"when they have the same ids (Note: they will be "
                      u"replaced *even* when published)."),
        default=False,
        required=False)


def make_import_log(context, importer, identifier='import_log'):
    # All of this need improvements
    log = context._getOb(identifier, None)
    if log is None:
        factory = context.manage_addProduct['silva.app.document']
        factory.manage_addDocument(identifier, 'Import log')
        log = context._getOb(identifier)
    version = log.get_editable()
    service = getUtility(IReferenceService)

    html = ['<h2> Import inside {0}</h2><ul>'.format(
            importer.root.get_title_or_id())]
    for reason, content in importer.getProblems():
        if IVersion.providedBy(content):
            content = content.get_silva_object()
        tag = unicode(uuid.uuid1())
        reference = service.new_reference(version, name=u"body link")
        reference.set_target(content)
        reference.add_tag(tag)
        html.extend(['<li><a class="link" reference="{0}">'.format(tag),
                     content.get_title_or_id(), '</a>: ',
                     reason, '</li>'])
    html.append('</ul>')
    version.body.save_raw_text(''.join(html) + unicode(version.body))
    notify(ObjectModifiedEvent(version))


class ImportForm(silvaforms.SMIForm):
    """Import the content of a file in Silva.
    """
    grok.adapts(Container, IContainer)
    grok.name('import')
    grok.require('silva.ManageSilvaContentSettings')

    label = _(u"Import and extract ZIP archive")
    fields = silvaforms.Fields(IImportFields)
    fields['archive'].fileNotSetLabel = _(
        u"Click the Upload button to select a file to import.")
    fields['archive'].fileSetLabel = _(
        u"Click the Upload button to replace the current file with a new file.")
    actions = silvaforms.Actions(silvaforms.CancelAction())

    @silvaforms.action(
        _(u"Import"),
        description=_(u"upload and import file"),
        implements=silvaforms.IDefaultAction,
        factory=silvaforms.ExtractedDecoratedAction,
        accesskey='ctrl+i')
    def import_file(self, data):
        # This need improvements and refactoing
        importer = IZipFileImporter(self.context)
        try:
            if importer.isFullmediaArchive(data['archive']):
                importer = importer.importFromZip(
                    data['archive'],
                    self.request,
                    {'replace_content': data.getDefault('replace'),
                     'ignore_top_level_content': data.getDefault('ignore_top_level')})
                make_import_log(self.context, importer)
                problems = importer.getProblems()
                if len(problems):
                    self.send_message(
                        _(u'Import is successful, but there are '
                          u'${many} problem(s).',
                          mapping={'many': len(problems)}),
                        type=u"error")
                else:
                    self.send_message(
                        _(u"Import succeeded."),
                        type=u"feedback")
            else:
                importer = IArchiveFileImporter(self.context)
                imported, failures = importer.importArchive(
                    data['archive'],
                    data.getDefault('asset_title'),
                    data.getDefault('sub_directories'),
                    data.getDefault('replace'))
                if imported:
                    self.send_message(
                        _('Imported ${succeeded}.',
                          mapping={'succeeded': ', '.join(imported)}),
                        type=u"feedback")
                if failures:
                    self.send_message(
                        _('Could not import: ${failed}.',
                          mapping={'failed': ', '.join(failures)}),
                        type=u"error")
        except zipfile.BadZipfile as error:
            self.send_message(
                _('Invalid import file: ${error}.',
                  mapping={'error': str(error)}),
                type=u"error")


class ImportMenu(MenuItem):
    grok.adapts(ContainerMenu, IContainer)
    grok.order(80)
    grok.require('silva.ManageSilvaContentSettings')
    name = _(u'Import')

    screen = ImportForm

    def available(self):
        return not IGhostAware.providedBy(self.content)



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.component import queryUtility
from zope.component.interfaces import IFactory
from zExceptions import NotFound

from silva.core.interfaces import IAddableContents, IIconResolver
from silva.core.interfaces import IContainer
from silva.core.views import views as silvaviews
from silva.translations import translate as _
from silva.ui.interfaces import IMenuItem
from silva.ui.menu import ExpendableMenuItem, ContentMenu
from silva.ui.rest import Screen, REST
from zeam.form import silva as silvaforms

from Products.Silva.ExtensionRegistry import extensionRegistry


class Adding(REST):
    grok.adapts(Screen, IContainer)
    grok.name('adding')
    grok.require('silva.ChangeSilvaContent')

    def publishTraverse(self, request, name):
        addables = IAddableContents(self.context).get_container_addables()
        if name in addables:
            factory = queryUtility(IFactory, name=name)
            if factory is not None:
                form = factory(self.context, request)
                # Set parent for security check.
                form.__name__ = name
                form.__parent__ = self
                return form
        # If it is not addable, it is not an addable.
        raise NotFound(name)


class AddableMenuItem(object):
    """A Addable MenuItem
    """
    grok.implements(IMenuItem)

    def __init__(self, addable):
        self.name = addable

    def available(self):
        return True

    def describe(self, page, path, actives):
        info = {'name': self.name,
                'screen': '/'.join((path, self.name))}
        if actives and actives[0].__name__ == self.name:
            info['active'] = True
        return info


class AddMenu(ExpendableMenuItem):
    grok.adapts(ContentMenu, IContainer)
    grok.order(15)
    grok.require('silva.ChangeSilvaContent')
    name = _('Add')
    screen = Adding

    def get_submenu_items(self):
        return map(
            AddableMenuItem,
            IAddableContents(self.content).get_authorized_addables())


class AddingInformation(silvaviews.Viewlet):
    grok.context(IContainer)
    grok.view(silvaforms.SMIAddForm)
    grok.viewletmanager(silvaforms.SMIFormPortlets)

    def update(self):
        addable = extensionRegistry.get_addable(self.view._content_type)
        self.name = addable.get('name')
        self.description = addable.get('doc')
        self.icon = IIconResolver(self.request).get_tag(
            identifier=addable.get('name'))



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.ftesting import smi_settings
from Products.Silva.testing import FunctionalLayer, Transaction


class ToolbarPublicationTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')

    def test_editor_publish_and_new_version(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('editor')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Editor only see a publish button (no version is waiting
        # approval by default)
        self.assertEqual(browser.inspect.toolbar, ['Publish'])
        self.assertEqual(browser.inspect.toolbar['Publish'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback('Version published.')

        # Now all the editor can do is create a new version
        self.assertEqual(browser.inspect.toolbar, ['New version'])
        self.assertEqual(browser.inspect.toolbar['New version'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback('New version created.')

        # After this you can publish this new version
        self.assertEqual(browser.inspect.toolbar, ['Publish'])

    def test_author_request_approval_and_approve(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Should have a button to request approval in the toolbar
        self.assertEqual(browser.inspect.toolbar, ['Request approval'])
        self.assertEqual(browser.inspect.toolbar['Request approval'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback(
            'Approval requested for immediate publication.')

        # Now all the author see in the toolbar is withdraw
        self.assertEqual(browser.inspect.toolbar, ['Withdraw request'])

        # Editor comes to publish the document
        browser.logout()
        browser.login('editor')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'author'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Should have a button to publish or reject
        self.assertEqual(browser.inspect.toolbar, ['Publish', 'Reject request'])
        self.assertEqual(browser.inspect.toolbar['Publish'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback('Version published.')

        # Document is publish, the only next possible step is to
        # create a new version
        self.assertEqual(browser.inspect.toolbar, ['New version'])

    def test_author_request_approval_and_withdraw(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Should have a button to request approval in the toolbar
        self.assertEqual(browser.inspect.toolbar, ['Request approval'])
        self.assertEqual(browser.inspect.toolbar['Request approval'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback(
            'Approval requested for immediate publication.')

        # Now all the author see in the toolbar is withdraw
        self.assertEqual(browser.inspect.toolbar, ['Withdraw request'])
        self.assertEqual(browser.inspect.toolbar['Withdraw request'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback('Withdrew request for approval.')

        # All the author can do now is to ask again
        self.assertEqual(browser.inspect.toolbar, ['Request approval'])

    def test_author_request_approval_and_reject(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Should have a button to request approval in the toolbar
        self.assertEqual(browser.inspect.toolbar, ['Request approval'])
        self.assertEqual(browser.inspect.toolbar['Request approval'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback(
            'Approval requested for immediate publication.')

        # Editor comes to reject the document
        browser.logout()
        browser.login('editor')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'author'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Should have a button to publish or reject
        self.assertEqual(browser.inspect.toolbar, ['Publish', 'Reject request'])
        self.assertEqual(browser.inspect.toolbar['Reject request'].click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        browser.macros.assertFeedback('Rejected request for approval.')

        # The editor can still choose to publish the document after
        self.assertEqual(browser.inspect.toolbar, ['Publish'])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ToolbarPublicationTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import FunctionalLayer
from Products.Silva.ftesting import smi_settings


class ReaderRootTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def test_root_roundtrip(self):
        """A reader can't add anything.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('reader')

        browser.open('/root/edit')
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.tabs, ['Content', 'Preview'])
        self.assertEqual(browser.inspect.views, ['View...'])

        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We go through the tabs
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Content'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Content'])


class AuthorRootTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'

    def test_root_roundtrip(self):
        """Test root as an author.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(
            browser.inspect.views,
            ['View...'])

        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We go through the tabs
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Content'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Content'])

    def test_root_settings(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertNotIn({'title': u'Container type'}, browser.inspect.form)


class EditorRootTestCase(AuthorRootTestCase):
    layer = FunctionalLayer
    user = 'editor'


class ChiefEditorRootTestCase(EditorRootTestCase):
    layer = FunctionalLayer
    user = 'chiefeditor'


class ManagerRootTestCase(ChiefEditorRootTestCase):
    layer = FunctionalLayer
    manager = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReaderRootTestCase))
    suite.addTest(unittest.makeSuite(AuthorRootTestCase))
    suite.addTest(unittest.makeSuite(EditorRootTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorRootTestCase))
    suite.addTest(unittest.makeSuite(ManagerRootTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import re
import lxml.html.diff

from five import grok
from zope.interface import Interface
from zope import schema
from zope.component import getMultiAdapter
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.traversing.browser import absoluteURL

from AccessControl.security import checkPermission

from Products.Silva.Versioning import VersioningError
from silva.core.interfaces import IVersion, IVersionManager
from silva.core.interfaces import IVersionedObject
from silva.core.interfaces import IPublicationWorkflow
from silva.ui.interfaces import IJSView
from silva.core.smi.content.publish import Publish
from silva.core.smi.content import ICompareResources
from silva.translations import translate as _
from silva.fanstatic import need
from silva.ui.rest import ActionResult
from silva.ui.rest import PageWithLayoutREST, PageREST
from zeam.form import silva as silvaforms
from zeam.form.silva.interfaces import IRemoverAction
from zeam.form.base.widgets import FieldWidget
from zeam.form.ztk.widgets.textline import TextLineField
from zope.cachedescriptors.property import Lazy


version_status_vocabulary = SimpleVocabulary([
        SimpleTerm(value="unapproved", title=_(u"Unapproved")),
        SimpleTerm(value="approved", title=_(u"Approved")),
        SimpleTerm(value="published", title=_(u"Published")),
        SimpleTerm(value="last_closed", title=_(u"Last closed")),
        SimpleTerm(value="closed", title=_(u"Closed"))])


class IPublicationStatusInfo(Interface):
    id = schema.TextLine(
        title=_(u'#'))
    modification_time = schema.Datetime(
        title=_('Modification'))
    publication_time = schema.Datetime(
        title=_('Publication'))
    expiration_time = schema.Datetime(
        title=_('Expiration'))
    last_author = schema.TextLine(
        title=_('Author'))
    version_status = schema.Choice(
        title=_('Status'),
        source=version_status_vocabulary)


class PublicationStatusInfo(grok.Adapter):
    grok.context(IVersion)
    grok.provides(IPublicationStatusInfo)

    def __init__(self, context):
        self.context = context
        self.manager = IVersionManager(self.context)

    @property
    def id(self):
        return self.context.getId()

    @property
    def modification_time(self):
        dt = self.manager.get_modification_datetime()
        if dt is not None:
            return dt.asdatetime()

    @property
    def publication_time(self):
        dt = self.manager.get_publication_datetime()
        if dt is not None:
            return dt.asdatetime()

    @property
    def expiration_time(self):
        dt = self.manager.get_expiration_datetime()
        if dt is not None:
            return dt.asdatetime()

    @property
    def last_author(self):
        author = self.manager.get_last_author()
        if author is not None:
            return author.fullname()

    @property
    def version_status(self):
        return self.manager.get_status()

    def delete(self):
        return self.manager.delete()

    def copy_for_editing(self):
        return self.manager.make_editable()


class CopyForEditingAction(silvaforms.Action):
    """Copy a version to use as new editable version.
    """
    title = _('Copy for editing')
    description = _('Create a new editable version of '
                    'the selected old one.')

    def __call__(self, form, selected, deselected):
        if len(selected) != 1:
            form.send_message(
                _(u"Please select one version to copy."),
                type='error')
            return silvaforms.FAILURE
        content = selected[0].getContentData().getContent()
        try:
            content.copy_for_editing()
        except VersioningError as e:
            form.send_message(e.reason, type='error')
            return silvaforms.FAILURE
        form.send_message(
            _("Reverted to previous version."),
            type='feedback')
        return silvaforms.SUCCESS


class ViewVersionAction(silvaforms.Action):
    """Preview a particular version.
    """
    title = _('View')
    description = _('View this particular version.')

    def __call__(self, form, selected, deselected):
        if len(selected) != 1:
            form.send_message(
                _(u"Please select one version to view it."),
                type='error')
            return silvaforms.FAILURE

        version = selected[0].getContentData().getContent().id

        def payload(rest):
            return {'content': {'ifaces': ['redirect'],
                                'screen': 'publish/view/' + version}}

        raise ActionResult(payload)


class ViewVersion(PageREST):
    grok.adapts(Publish, IVersionedObject)
    grok.require('silva.ReadSilvaContent')
    grok.name('view')

    version = None

    def publishTraverse(self, request, name):
        match = re.compile('^(\d+)$').match(name)
        if match:
            version = self.context._getOb(match.group(1), None)
            if IVersion.providedBy(version):
                self.version = version
                self.__name__ = '/'.join((self.__name__, name))
                return self
        return super(ViewVersion, self).publishTraverse(request, name)

    def payload(self):
        assert self.version is not None
        view = getMultiAdapter(
            (self.version, self.request), IJSView, name='content-preview')
        return view(self)


class CompareVersionAction(silvaforms.Action):
    """Compare two different versions.
    """
    title = _('Compare')
    descripton = _('Select and compare two different versions.')

    def available(self, form):
        return len(form.lines) > 1

    def __call__(self, form, selected, deselected):
        if len(selected) != 2:
            form.send_message(
                _(u"Select exactly two different versions to compare them."),
                type='error')
            return silvaforms.FAILURE
        id1 = selected[0].getContentData().getContent().id
        id2 = selected[1].getContentData().getContent().id

        def payload(rest):
            return {'content': {'ifaces': ['redirect'],
                                'screen': 'publish/compare/%s-%s' % (id1, id2)}}

        raise ActionResult(payload)


class CompareVersionScreen(PageREST):
    """Screen to compare two versions
    """
    grok.adapts(Publish, IVersionedObject)
    grok.require('silva.ReadSilvaContent')
    grok.name('compare')

    version1 = None
    version2 = None

    def publishTraverse(self, request, name):
        match = re.compile('^(\d+)-(\d+)$').match(name)
        if match:
            version1 = self.context._getOb(match.group(1), None)
            version2 = self.context._getOb(match.group(2), None)
            if IVersion.providedBy(version1) and IVersion.providedBy(version2):
                self.version1 = version1
                self.version2 = version2
                self.__name__ = '/'.join((self.__name__, name))
                return self
        return super(CompareVersionScreen, self).publishTraverse(request, name)

    def payload(self):
        url = absoluteURL(self, self.request) + '/view'
        return {'ifaces': ['preview'], 'html_url': url}


class CompareVersionPage(PageWithLayoutREST):
    grok.adapts(CompareVersionScreen, IVersionedObject)
    grok.require('silva.ReadSilvaContent')
    grok.name('view')

    def content(self):
        assert isinstance(self.__parent__, CompareVersionScreen)
        assert self.__parent__.version1 is not None
        assert self.__parent__.version2 is not None

        version1_view = getMultiAdapter(
            (self.context, self.request), name='content.html')
        version1_view.content = self.__parent__.version1
        version1_html = version1_view()

        version2_view = getMultiAdapter(
            (self.context, self.request), name='content.html')
        version2_view.content = self.__parent__.version2
        version2_html = version2_view()

        need(ICompareResources)
        return lxml.html.diff.htmldiff(version2_html, version1_html)


class DeleteVersionAction(silvaforms.Action):
    """Permanently remove version.
    """
    grok.implements(IRemoverAction)
    title = _("Delete")
    description = _("There's no undo.")
    htmlAttributes = {
        'data-confirmation-title': _(
            u'Delete a version'),
        'data-confirmation': _(
            u'Deletion is premanent. Are you sure ?')}

    def available(self, form):
        return (len(form.lines) > 1 and
                bool(checkPermission('silva.ApproveSilvaContent', form.context)))

    def __call__(self, form, selected, deselected):
        if not len(deselected):
            form.send_message(
                _(u"Cannot delete all versions."),
                type='error')
            return silvaforms.FAILURE
        if not len(selected):
            form.send_message(
                _(u"No version selected, nothing deleted."),
                type='error')
            return silvaforms.FAILURE
        for line in selected:
            try:
                line.getContentData().getContent().delete()
            except VersioningError as e:
                form.send_message(e.reason, type='error')
        form.ignoreRequest = True # Don't keep the selected lines selected
        form.send_message(_(u"Version(s) deleted."), type='feedback')
        return silvaforms.SUCCESS


class VersioningStateWidget(FieldWidget):
    MODE = 'versioning_state'
    grok.adapts(TextLineField, Interface, Interface)
    grok.name(MODE)

    @Lazy
    def versioning_state(self):
        version = self.form.getContent().context
        manager = IVersionManager(version)
        return str(manager.get_status())

    @Lazy
    def link_attributes(self):
        version = self.form.getContent().context
        content = version.get_silva_object()
        return {
            'href': self.form.parent.getComposedForm().get_content_path(content),
            'rel': 'publish/view/' + version.id,
            'class': 'open-screen'
        }


class PublicationStatusTableForm(silvaforms.SMISubTableForm):
    """ Manage versions.
    """
    grok.context(IVersionedObject)
    grok.view(Publish)
    grok.order(100)

    label = _(u"Manage versions")
    mode = silvaforms.DISPLAY

    ignoreRequest = True
    ignoreContent = False

    batchSize = 10
    batchItemFactory = IPublicationStatusInfo

    tableFields = silvaforms.Fields(IPublicationStatusInfo)
    tableFields['id'].mode = VersioningStateWidget.MODE

    tableActions = silvaforms.TableMultiActions(
        DeleteVersionAction(identifier='delete'),
        CopyForEditingAction(identifier='copy'),
        ViewVersionAction(identifier='view'),
        CompareVersionAction(identifier='compare'))

    def getItemIdentifier(self, item, position):
        return str(item.id)

    def getItems(self):
        versions = IPublicationWorkflow(self.context).get_versions(False)
        versions.sort(key=lambda v: int(v.getId()), reverse=True)
        return versions




# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt


from five import grok
from zope.interface import Interface
from silva.core.interfaces import IVersionedObject
from silva.translations import translate as _
from silva.ui.menu import ContentMenu, ExpendableMenuItem
from silva.ui.rest import Screen
from zeam.form import silva as silvaforms


class Publish(silvaforms.SMIComposedForm):
    """Publish tab.
    """
    grok.adapts(Screen, IVersionedObject)
    grok.require('silva.ChangeSilvaContent')
    grok.name('publish')

    label = _('Publication')

    def get_menu_parent(self):
        parent = super(Publish, self).get_menu_parent()
        parent['screen'] = 'content' # Containers have no publish screen
        return parent


class PublishMenu(ExpendableMenuItem):
    grok.adapts(ContentMenu, IVersionedObject)
    grok.require('silva.ChangeSilvaContent')
    grok.order(30)
    name = _('Publish')

    screen = Publish


class IPublicationFields(Interface):
    """ Interface to register autofields for request approval
    """



# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.interface import Interface
from zope import schema

from silva.core.interfaces import IHTTPHeadersSettings
from silva.core.interfaces import ISilvaObject
from silva.core.smi.settings import SettingsMenu, Settings
from silva.translations import translate as _
from silva.ui.menu import MenuItem
from zeam.form import silva as silvaforms


class IHTTPHeadersSettingsFields(Interface):

    http_disable_cache = schema.Bool(
        title=_(u'Prevent item from ever being cached'),
        description=_(u'Add headers to prevent HTTP proxies to cache item.'),
        required=False,
        default=False)
    http_max_age = schema.Int(
        title=_(u'Maximum caching time'),
        description=_(u'If the item is cached, this is the maximum time '
                      u'in seconds it can be cached by an HTTP proxy. '
                      u'Minimal time is 30 seconds.'),
        default=84600,
        min=30,
        required=True)
    http_last_modified = schema.Bool(
        title=_(u'Include a Last-Modified header in the response'),
        description=_(
            u'This information is used to control the caching '
            u'of an item. If not specified, an HTTP proxy will fetch '
            u'the item as often as possible.'),
        required=False,
        default=True)


def cache_not_disabled(form):
    return not form.getContent().http_disable_cache


class HTTPHeadersSettingsForm(silvaforms.SMIForm):
    grok.adapts(Settings, ISilvaObject)
    grok.require('silva.ManageSilvaContentSettings')

    label = _('HTTP Caching settings')
    fields = silvaforms.Fields(IHTTPHeadersSettingsFields)
    fields['http_max_age'].available = cache_not_disabled
    actions = silvaforms.Actions(
        silvaforms.CancelAction(),
        silvaforms.EditAction())
    ignoreRequest = False
    ignoreContent = False

    def update(self):
        self.setContentData(IHTTPHeadersSettings(self.context, None))


class CachingMenu(MenuItem):
    grok.adapts(SettingsMenu, ISilvaObject)
    grok.order(10000)
    grok.require('silva.ManageSilvaContentSettings')
    name = _(u'HTTP Caching')
    screen = HTTPHeadersSettingsForm

    def available(self):
        return IHTTPHeadersSettings(self.content, None) is not None



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import FunctionalLayer
from Products.Silva.ftesting import smi_settings
from silva.core.references.reference import get_content_id


class AuthorContentTestCase(unittest.TestCase):
    """For each role make each content in a folder.
    """
    layer = FunctionalLayer
    username = 'author'

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')

    def test_image(self):
        """Test create / edit / remove an image.
        """
        browser = self.layer.get_web_browser(smi_settings)

        image = self.layer.get_fixture('torvald.jpg')
        browser.login(self.username, self.username)
        self.assertEqual(browser.open('/root/edit'), 200)
        browser.macros.create(
            'Silva Image', id='image', title='Torvald', file=image)
        self.assertEqual(
            browser.inspect.folder_listing, ['index', 'image'])

        # The user should by the last author on the content and container.
        self.assertEqual(
            self.root.sec_get_last_author_info().userid(),
            self.username)
        self.assertEqual(
            self.root.image.sec_get_last_author_info().userid(),
            self.username)

        # Visit the edit page
        self.assertEqual(
            browser.inspect.folder_listing['image'].click(),
            200)
        self.assertEqual(browser.location, '/root/image/edit/tab_edit')

        # Change title
        form = browser.get_form('silvaObjects')
        self.assertEqual(
            form.get_control('field_image_title').value,
            'Torvald')
        form.get_control('field_image_title').value = u'Picture of Torvald'
        form.get_control('submit:method').click()
        self.assertEqual(browser.inspect.feedback, ['Changes saved.'])

        # Change format
        form = browser.get_form('editform.scaling')
        self.assertEqual(form.get_control('field_web_format').value, 'JPEG')
        form.get_control('field_web_format').value = 'PNG'
        form.get_control('scale_submit:method').click()
        self.assertEqual(
            browser.inspect.feedback,
            ['Scaling and/or format changed.'])

        # Change scaling
        form = browser.get_form('editform.scaling')
        form.get_control('field_web_scaling').value = '100x200'
        form.get_control('scale_submit:method').click()
        self.assertEqual(
            browser.inspect.feedback,
            ['Scaling and/or format changed.'])

        # Change image
        form = browser.get_form('editform.upload')
        form.get_control('field_file').value = image
        form.get_control('upload_submit:method').click()
        self.assertEqual(
            browser.inspect.feedback,
            ['Image updated.'])

        self.assertEqual(
            browser.inspect.breadcrumbs,
            ['root', 'Picture of Torvald'])
        browser.inspect.breadcrumbs['root'].click()
        browser.macros.delete('image')

    def test_file(self):
        """Test create/remove a file.
        """
        browser = self.layer.get_web_browser(smi_settings)

        image = test_filename('test.txt')
        browser.login(self.username, self.username)
        self.assertEqual(browser.open('/root/edit'), 200)
        browser.macros.create(
            'Silva File', id='file', title='Text File', file=image)
        self.assertEqual(
            browser.inspect.folder_listing, ['index', 'file'])

        # The user should by the last author on the content and container.
        self.assertEqual(
            self.root.sec_get_last_author_info().userid(),
            self.username)
        self.assertEqual(
            self.root.file.sec_get_last_author_info().userid(),
            self.username)

        # Visit the edit page
        self.assertEqual(
            browser.inspect.folder_listing['file'].click(),
            200)
        self.assertEqual(browser.url, '/root/file/edit/tab_edit')
        self.assertEqual(browser.inspect.breadcrumbs, ['root', 'Text File'])
        browser.inspect.breadcrumbs['root'].click()
        browser.macros.delete('file')



def test_suite():
    suite = unittest.TestSuite()
    return suite
    suite.addTest(unittest.makeSuite(AuthorContentTestCase))




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.ftesting import rest_settings
from Products.Silva.testing import FunctionalLayer, Transaction
from silva.core.cache.memcacheutils import Reset
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


MOCKUP_ICON = u'http://localhost/root/++resource++icon-Mockup-VersionedContent.png'


class AuthorFolderActionsTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'
    access = 'write'
    maxDiff = None

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login(self.user)
        self.get_id = getUtility(IIntIds).getId
        with Reset():
            with Transaction():
                factory = self.root.manage_addProduct['Silva']
                factory.manage_addFolder('folder', 'Folder')
                factory = self.root.folder.manage_addProduct['Silva']
                factory.manage_addMockupVersionedContent('document', 'Document')
                factory.manage_addFolder('information', 'Information')

    def test_delete_unexisting_content(self):
        """Test delete an unexisting content.
        """
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.delete',
                    method='POST',
                    form={'content': '42'}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertIsInstance(browser.json, dict)
            self.assertEquals(
                browser.json,
                {u'content': {
                        u'actions': {},
                        u'ifaces': [u'listing-changes']},
                 u'notifications': [{
                            u'category': u'error',
                            u'message': u'1 contents could not be found (they probably have been deleted).'}]})

    def test_delete_unpublished_folder(self):
        """Test deleting an unpublished folder.
        """
        information_id = self.get_id(self.root.folder.information)
        folder_id = self.get_id(self.root.folder)
        root_id = self.get_id(self.root)
        zope_id = self.get_id(self.root.__parent__)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.delete',
                    method='POST',
                    form={'content': folder_id}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertIsInstance(browser.json, dict)
            self.assertEquals(
                browser.json,
                {u'content': {
                        u'actions': {u'remove': [folder_id]},
                        u'ifaces': [u'listing-changes']},
                 u'navigation': {
                        u'invalidation': [{
                                u'action': u'remove',
                                u'info': {
                                    u'target': u'nav%s' % information_id}}, {
                                u'action': u'remove',
                                u'info': {
                                    u'target': u'nav%s' % folder_id}}, {
                                u'action': u'update',
                                u'info': {
                                    u'parent': u'nav%s' % zope_id,
                                    u'position': -1,
                                    u'target': u'nav%s' % root_id}}]},
                 u'notifications': [{
                            u'category': u'feedback',
                            u'message': u'Deleted "Folder".'}]})

    def test_copy_unpublished_folder(self):
        """Test copying an unpublished folder.
        """
        folder_id = self.get_id(self.root.folder.information)
        root_id = self.get_id(self.root)
        zope_id = self.get_id(self.root.__parent__)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.paste',
                    method='POST',
                    form={'copied': folder_id}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertIsInstance(browser.json, dict)
            copy = self.root._getOb('information', None)
            self.assertIsNot(copy, None)
            self.assertIsNot(self.root.folder._getOb('information', None), None)
            copy_id = self.get_id(copy)
            copy_date = copy.get_modification_datetime()
            self.assertIsNot(copy_date, None)
            self.assertEquals(
                browser.json,
                {u'content': {
                        u'actions': {
                            u'add': {
                                u'publishables': [{
                                        u'access': self.access,
                                        u'author': self.user,
                                        u'icon': u'silva_folder',
                                        u'id': copy_id,
                                        u'identifier': u'information',
                                        u'ifaces': [u'container'],
                                        u'modified': copy_date.strftime('%y/%m/%d %H:%M'),
                                        u'moveable': True,
                                        u'path': u'information',
                                        u'position': -1,
                                        u'status_next': None,
                                        u'status_public': None,
                                        u'title': u'Information'}]}},
                              u'ifaces': [u'listing-changes']},
                 u'navigation': {
                        u'invalidation': [{
                                u'action': u'add',
                                u'info': {
                                    u'parent': u'nav%s' % root_id,
                                    u'position': -1,
                                    u'target': u'nav%s' % copy_id}}, {
                                u'action': u'update',
                                u'info': {
                                    u'parent': u'nav%s' % zope_id,
                                    u'position': -1,
                                    u'target': u'nav%s' % root_id}}]},
                 u'notifications': [{
                            u'category': u'feedback',
                            u'message': u'Pasted as a copy "Information".'}]})

    def test_copy_unpublished_document(self):
        """Test copying an unpublished document.
        """
        document_id = self.get_id(self.root.folder.document)
        root_id = self.get_id(self.root)
        zope_id = self.get_id(self.root.__parent__)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.paste',
                    method='POST',
                    form={'copied': document_id}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertIsInstance(browser.json, dict)
            copy = self.root._getOb('document', None)
            self.assertIsNot(copy, None)
            self.assertIsNot(self.root.folder._getOb('document', None), None)
            copy_id = self.get_id(copy)
            copy_date = copy.get_modification_datetime()
            self.assertIsNot(copy_date, None)
            self.assertEquals(
                browser.json,
                {u'content': {
                        u'actions': {
                            u'add': {
                                u'publishables': [{
                                        u'access': self.access,
                                        u'author': self.user,
                                        u'icon': MOCKUP_ICON,
                                        u'id': copy_id,
                                        u'identifier': u'document',
                                        u'ifaces': [u'versioned'],
                                        u'modified': copy_date.strftime('%y/%m/%d %H:%M'),
                                        u'moveable': True,
                                        u'path': u'document',
                                        u'position': -1,
                                        u'status_next': u'draft',
                                        u'status_public': None,
                                        u'title': u'Document'}]}},
                              u'ifaces': [u'listing-changes']},
                 u'navigation': {
                        u'invalidation': [{
                                u'action': u'update',
                                u'info': {
                                    u'parent': u'nav%s' % zope_id,
                                    u'position': -1,
                                    u'target': u'nav%s' % root_id}}]},
                 u'notifications': [{
                            u'category': u'feedback',
                            u'message': u'Pasted as a copy "Document".'}]})

    def test_moving_unpublished_document(self):
        """Test moving an unpublished document using the REST interface.
        """
        document_id = self.get_id(self.root.folder.document)
        root_id = self.get_id(self.root)
        folder_id = self.get_id(self.root.folder)
        zope_id = self.get_id(self.root.__parent__)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.paste',
                    method='POST',
                    form={'cutted': document_id}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertIsInstance(browser.json, dict)
            self.assertIsNot(self.root._getOb('document', None), None)
            self.assertIs(self.root.folder._getOb('document', None), None)
            document_date = self.root.document.get_modification_datetime()
            folder_date = self.root.folder.get_modification_datetime()
            self.assertEquals(
                browser.json,
                {u'content': {
                        u'actions': {
                            u'add': {
                                u'publishables': [{
                                        u'access': self.access,
                                        u'author': self.user,
                                        u'icon': MOCKUP_ICON,
                                        u'id': document_id,
                                        u'identifier': u'document',
                                        u'ifaces': [u'versioned'],
                                        u'modified': document_date.strftime('%y/%m/%d %H:%M'),
                                        u'moveable': True,
                                        u'path': u'document',
                                        u'position': -1,
                                        u'status_next': u'draft',
                                        u'status_public': None,
                                        u'title': u'Document'}]},
                            u'update': [{
                                    u'access': self.access,
                                    u'author': self.user,
                                    u'icon': u'silva_folder',
                                    u'id': folder_id,
                                    u'identifier': u'folder',
                                    u'ifaces': [u'container'],
                                    u'modified': folder_date.strftime('%y/%m/%d %H:%M'),
                                    u'moveable': True,
                                    u'path': u'folder',
                                    u'position': 1,
                                    u'status_next': None,
                                    u'status_public': None,
                                    u'title': u'Folder'}]},
                        u'ifaces': [u'listing-changes']},
                 u'navigation': {
                        u'invalidation': [{
                                u'action': u'update',
                                u'info': {u'parent': u'nav%s' % root_id,
                                          u'position': 1,
                                          u'target': u'nav%s' % folder_id}}, {
                                u'action': u'update',
                                u'info': {
                                    u'parent': u'nav%s' % zope_id,
                                    u'position': -1,
                                    u'target': u'nav%s' % root_id}}]},
                 u'notifications': [{
                            u'category': u'feedback',
                            u'message': u'Moved "Document".'}]})

    def test_publish_document(self):
        """Test publishing a document from the REST interface.
        """
        document_id = self.get_id(self.root.folder.document)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/folder/++rest++silva.ui.listing.publish',
                    method='POST',
                    form={'content': document_id}),
                401)


class EditorFolderActionsTestCase(AuthorFolderActionsTestCase):
    user = 'editor'
    access = 'publish'

    def test_publish_document(self):
        """Test publishing a document from the REST interface.
        """
        document_id = self.get_id(self.root.folder.document)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/folder/++rest++silva.ui.listing.publish',
                    method='POST',
                    form={'content': document_id}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertIsInstance(browser.json, dict)
            document_date = self.root.folder.document.get_modification_datetime()
            self.assertEqual(
                browser.json,
                {u'content': {
                        u'actions': {
                            u'update': [{
                                    u'access': self.access,
                                    u'author': self.user,
                                    u'icon': MOCKUP_ICON,
                                    u'id': document_id,
                                    u'identifier': u'document',
                                    u'ifaces': [u'versioned'],
                                    u'modified': document_date.strftime('%y/%m/%d %H:%M'),
                                    u'moveable': True,
                                    u'path': u'folder/document',
                                    u'position': 1,
                                    u'status_next': None,
                                    u'status_public': u'published',
                                    u'title': u'Document'}]},
                        u'ifaces': [u'listing-changes']},
                 u'notifications': [{
                            u'category': u'feedback',
                            u'message': u'Published "Document".'}]})


class ChiefEditorFolderActionsTestCase(EditorFolderActionsTestCase):
    user = 'chiefeditor'
    access = 'manage'


class ManagerFolderActionsTestCase(ChiefEditorFolderActionsTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AuthorFolderActionsTestCase))
    suite.addTest(unittest.makeSuite(EditorFolderActionsTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorFolderActionsTestCase))
    suite.addTest(unittest.makeSuite(ManagerFolderActionsTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.interface import Interface
from zope.component import getMultiAdapter, getUtility

from silva.core.interfaces import IViewableObject, IDirectlyRendered
from silva.core.views.interfaces import ISilvaURL
from silva.translations import translate as _
from silva.ui.interfaces import IJSView, IUIService
from silva.ui.interfaces import IUIScreen
from silva.ui.menu import ContentMenu, MenuItem
from silva.ui.rest import PageREST, Screen


class PreviewJSView(grok.MultiAdapter):
    """Preview JSView.
    """
    grok.provides(IJSView)
    grok.adapts(Interface, Interface)
    grok.name('content-preview')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, screen):
        service = getUtility(IUIService)
        url = getMultiAdapter(
            (self.context, self.request),
            ISilvaURL).url(preview=True, host=service.preview_url or None)
        return {"ifaces": ["preview"],
                "html_url": url}


class DirectlyRenderedPreview(grok.MultiAdapter):
    """Preview JSView for directly rendered contents.
    """
    grok.provides(IJSView)
    grok.adapts(IDirectlyRendered, Interface)
    grok.name('content-preview')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, screen):
        view = getMultiAdapter(
            (self.context, self.request),
            name='content.html')
        return {"ifaces": ["preview"],
                "html": view()}


class IPreviewScreen(IUIScreen):
    """Marker used to mark edit tab.
    """


class Preview(PageREST):
    grok.adapts(Screen, IViewableObject)
    grok.implements(IPreviewScreen)
    grok.name('preview')
    grok.require('silva.ReadSilvaContent')

    def payload(self):
        view = getMultiAdapter(
            (self.context, self.request),
            IJSView,
            name='content-preview')
        return view(self)


class PreviewMenu(MenuItem):
    grok.adapts(ContentMenu, IViewableObject)
    grok.order(11)
    name = _('Preview')
    description = _(u'Preview this item inside the management interface.')
    screen = 'preview'
    interface = IPreviewScreen
    accesskey = u';'



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.component import getMultiAdapter, getUtility

from silva.core.interfaces import IViewableObject
from silva.core.views.interfaces import ISilvaURL
from silva.translations import translate as _
from silva.ui.interfaces import IUIService
from silva.ui.menu import LinkMenuItem, ViewMenu


class DisplayMenu(LinkMenuItem):
    grok.adapts(ViewMenu, IViewableObject)
    grok.order(20)
    name = _('View...')
    description = _(u'See the public view of this content in a new tab/window.')
    accesskey = u'.'

    def get_url(self, context, request):
        service = getUtility(IUIService)
        url = getMultiAdapter(
            (context, request),
            ISilvaURL).url(host=service.public_url or None)
        return url



# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.interface import Interface
from zope.traversing.browser import absoluteURL
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from infrae.rest import RESTWithTemplate
from silva.core import conf as silvaconf
from silva.core.interfaces import ISilvaObject
from silva.core.smi.userinfo import UserSettingsMenu
from silva.fanstatic import need
from silva.translations import translate as _
from silva.ui.menu import LinkMenuItem
from silva.ui.rest.helper import ResourcesProvider


class IAboutResources(IDefaultBrowserLayer):
    silvaconf.resource('about.css')


class About(RESTWithTemplate):
    grok.context(ISilvaObject)
    grok.name('silva.core.smi.about')

    def GET(self):
        root = self.context.get_root()
        self.version = root.get_silva_software_version()
        need(IAboutResources)
        data = {
            'content': {
                'ifaces': ['text-overlay'],
                'html': self.template.render(self)
            }}
        ResourcesProvider(self, self.request)(self, data)
        return self.json_response(data)


class AboutMenu(LinkMenuItem):
    grok.adapts(UserSettingsMenu, Interface)
    grok.order(0)
    name = _('About')
    icon = 'about'
    trigger = 'text-overlay'

    def get_url(self, context, request):
        return '{0}/++rest++silva.core.smi.about'.format(
            absoluteURL(context, request))



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from silva.ui.menu import ContentMenu, ExpendableMenuItem
from silva.ui.rest import Screen
from silva.core.interfaces import ISilvaObject
from silva.translations import translate as _
from zeam.form import silva as silvaforms


class Settings(silvaforms.SMIComposedForm):
    grok.adapts(Screen, ISilvaObject)
    grok.require('silva.ChangeSilvaContent')
    grok.name('settings')

    label = _('Settings')



class SettingsMenu(ExpendableMenuItem):
    grok.adapts(ContentMenu, ISilvaObject)
    grok.require('silva.ChangeSilvaContent')
    grok.order(80)
    name = _('Settings')
    screen = Settings




# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import operator

from five import grok
from zope import interface, schema, component
from zope.cachedescriptors.property import CachedProperty

from silva.core.cache.store import SessionStore
from silva.core.interfaces import IAccessSecurity, IAuthorizationManager
from silva.core.interfaces import ISilvaObject
from silva.core.interfaces import UnauthorizedRoleAssignement
from silva.core.interfaces import role_vocabulary, authenticated_role_vocabulary
from silva.core.services.interfaces import IMemberService, MemberLookupError
from silva.core.smi.settings import SettingsMenu, Settings
from silva.translations import translate as _
from silva.ui.menu import MenuItem
from zeam.form import silva as silvaforms

USER_STORE_KEY = 'lookup user'


class Access(silvaforms.SMIComposedForm):
    """Control access to Silva.
    """
    grok.adapts(Settings, ISilvaObject)
    grok.name('access')
    grok.require('silva.ChangeSilvaAccess')

    label = _(u"Manage access to content")
    description = _(u"This screen lets you authorize or revoke access "
                    u"to this item and all content it contains.")


class AccessMenu(MenuItem):
    grok.adapts(SettingsMenu, ISilvaObject)
    grok.order(10)
    grok.require('silva.ChangeSilvaAccess')
    name = _(u'Access')
    screen = Access


class LookupUserPopupAction(silvaforms.PopupAction):
    title = _(u"Lookup users...")
    description = _(u"Search for users to assign them roles")
    action = 'silva.core.smi.lookupuser'
    accesskey = u'ctrl+l'


class ILookupUserSchema(interface.Interface):
    """Lookup a new user
    """
    user = schema.TextLine(
        title=_(u"Username"),
        description=_(u"Username or part of the username to lookup"),
        required=True)


class LookupUserAction(silvaforms.Action):
    grok.implements(silvaforms.IRESTCloseOnSuccessAction,
                    silvaforms.IRESTRefreshAction,
                    silvaforms.IDefaultAction)
    refresh = 'form.userrole.lookupuserresultform'

    title = _(u"Lookup user")

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            form.send_message(
                _(u"Sorry, there were errors."),
                type="error")
            return silvaforms.FAILURE
        username = data['user'].strip()

        service = component.getUtility(IMemberService)
        users = set()
        new_users = set()
        try:
            for member in service.find_members(username, location=form.context):
                userid = member.userid()
                users.add(userid)
                new_users.add(userid)
        except MemberLookupError as error:
            form.send_message(error.args[0], type="error")
            return silvaforms.FAILURE

        store = SessionStore(form.request)
        if new_users:
            users = store.get(USER_STORE_KEY, set()).union(users)
            store.set(USER_STORE_KEY, users)
            form.send_message(
                _(u"Found ${count} users: ${users}.",
                  mapping={'count': len(new_users),
                           'users': u', '.join(new_users)}),
                type="feedback")
        else:
            form.send_message(
                _(u"No matching users were found."),
                type="error")
            return silvaforms.FAILURE
        return silvaforms.SUCCESS


class LookupUserForm(silvaforms.RESTPopupForm):
    """Form to manage default permission needed to see the current
    content.
    """
    grok.name('silva.core.smi.lookupuser')
    grok.context(ISilvaObject)

    label = _(u"Lookup users")
    description = _(u"Search for users to assign them roles. Enter as least "
                    u"two characters. Looukup entire "
                    u"names to limit the number of results.")
    fields = silvaforms.Fields(ILookupUserSchema)
    actions = silvaforms.Actions(
        silvaforms.CancelAction(),
        LookupUserAction())


class UserRole(silvaforms.SMISubFormGroup):
    grok.context(ISilvaObject)
    grok.order(20)
    grok.view(Access)


class IGrantRoleSchema(interface.Interface):
    """A role for a user.
    """
    role = schema.Choice(
        title=_(u"Role to grant:"),
        source=role_vocabulary,
        required=True)


class GrantAccessAction(silvaforms.Action):

    title = _(u"Grant role")
    description = _(u"Grant the selected role to selected users(s)")

    def available(self, form):
        return len(form.lines) != 0

    def __call__(self, form, authorization, line):
        data, errors = form.extractData(form.fields)
        if errors:
            return silvaforms.FAILURE
        role = data['role']
        if not role:
            return form.revoke(authorization, line)
        mapping = {'role': role,
                   'username': authorization.identifier}
        try:
            if authorization.grant(role):
                form.send_message(
                    _('Role "${role}" granted to user "${username}".',
                      mapping=mapping),
                    type="feedback")
            else:
                form.send_message(
                    _('User "${username}" already has the role "${role}".',
                      mapping=mapping),
                    type="error")
        except UnauthorizedRoleAssignement as error:
            form.send_message(
                _(u'Sorry, you are not allowed to grant the role "${role}" '
                  u'to user "${username}": ${reason}.',
                  mapping={'role': error.role,
                           'username': error.identifier,
                           'reason': error.reason}),
                type="error")
        return silvaforms.SUCCESS


class RevokeAccessAction(silvaforms.Action):
    grok.implements(silvaforms.IRemoverAction)

    title = _(u"Revoke role")
    description=_(u"Revoke the role of selected user(s)")

    def available(self, form):
        return reduce(
            operator.or_,
            [False] + map(lambda l: l.getContent().local_role is not None,
                          form.lines))

    def __call__(self, form, authorization, line):
        try:
            role = authorization.local_role
            username = authorization.identifier
            if authorization.revoke():
                form.send_message(
                    _(u'Removed role "${role}" from user "${username}".',
                      mapping={'role': role,
                               'username': username}),
                    type="feedback")
            else:
                form.send_message(
                    _(u'User "${username}" doesn\'t have any local role.',
                      mapping={'username': username}),
                    type="error")
        except UnauthorizedRoleAssignement as error:
            form.send_message(
                _(u'Sorry, you are not allowed to remove the role "${role}" '
                  u'from user "${username}": ${reason}.',
                  mapping={'role': error.role,
                           'username': error.identifier,
                           'reason': error.reason}),
                type="error")
        return silvaforms.SUCCESS


class IUserAuthorization(interface.Interface):

    identifier = schema.TextLine(
        title=_(u"Identifier"))
    name = schema.TextLine(
        title=_(u"Username"))
    email = schema.TextLine(
        title=_(u'Email'))
    acquired_role = schema.Choice(
        title=_(u"Role defined above"),
        source=role_vocabulary,
        required=False)
    local_role = schema.Choice(
        title=_(u"Role defined here"),
        source=role_vocabulary,
        required=False)


def show_username(form):
    if not hasattr(form, 'show_username'):
        form = form.parent
    return form.show_username

def show_email(form):
    if not hasattr(form, 'show_email'):
        form = form.parent
    return form.show_email


class UserAccessForm(silvaforms.SMISubTableForm):
    """Form to give/revoke access to users.
    """
    grok.context(ISilvaObject)
    grok.order(20)
    grok.view(UserRole)

    label = _(u"User roles")
    emptyDescription = _(u'No roles have been assigned.')
    ignoreContent = False
    ignoreRequest = True
    mode = silvaforms.DISPLAY
    fields = silvaforms.Fields(IGrantRoleSchema)
    fields['role'].mode = silvaforms.INPUT
    fields['role'].ignoreRequest = False
    fields['role'].ignoreContent = True
    fields['role'].available = lambda form: len(form.lines) != 0
    tableFields = silvaforms.Fields(IUserAuthorization)
    tableFields['identifier'].mode = 'silva.icon'
    tableFields['identifier'].available = lambda form: not show_username(form)
    tableFields['name'].available = show_username
    tableFields['email'].available = show_email
    tableActions = silvaforms.TableActions(
        GrantAccessAction(),
        RevokeAccessAction())

    show_username = False
    show_email = False

    def update(self):
        service = component.getUtility(IMemberService)
        if service.get_display_usernames():
            self.show_username = True
        if service.get_display_emails():
            self.show_email = True

    def getItems(self):
        access = IAuthorizationManager(self.context)
        authorizations = access.get_defined_authorizations().items()
        authorizations.sort(key=operator.itemgetter(0))
        return filter(lambda auth: auth.type == 'user',
                      map(operator.itemgetter(1), authorizations))


class LookupUserResultForm(UserAccessForm):
    """Form to give/revoke access to users.
    """
    grok.order(10)

    label = _(u"User clipboard")
    emptyDescription = _(u'In order to assign roles to users, first lookup '
                         u'users so they\'re listed on this user clipboard. '
                         u'Then assign the users roles. Any users on the '
                         u'clipboard will remain here when you move to other '
                         u'areas of the site.')
    actions = silvaforms.Actions()
    tableActions = silvaforms.TableActions(GrantAccessAction())

    @CachedProperty
    def store(self):
        return SessionStore(self.request)

    def getItems(self):
        user_ids = self.store.get(USER_STORE_KEY, set())
        if user_ids:
            access = IAuthorizationManager(self.context)
            authorizations = access.get_authorizations(user_ids).items()
            authorizations.sort(key=operator.itemgetter(0))
            return filter(lambda auth: auth.type == 'user',
                          map(operator.itemgetter(1), authorizations))
        return []

    @silvaforms.action(
        _(u"Clear clipboard"),
        description=_(u"clear the user clipboard"),
        available=lambda form: len(form.lines) != 0,
        implements=silvaforms.IRemoverAction)
    def clear(self):
        self.store.set(USER_STORE_KEY, set())

    actions += LookupUserPopupAction()


class IAccessMinimumRoleSchema(interface.Interface):
    """A role for a user.
    """
    acquired = schema.Bool(
        title=_(u"Is the role acquired from above?"),
        required=False)
    minimum_role = schema.Choice(
        title=_(u"Minimum role"),
        description=_(u"Minimum required role needed to access this item"),
        source=authenticated_role_vocabulary,
        required=False)


class AccessPermissionForm(silvaforms.SMISubForm):
    """Form to manage default permission needed to see the current
    content.
    """
    grok.context(ISilvaObject)
    grok.order(100)
    grok.view(Access)

    label = _(u"Public view access restriction")
    description = _(u"Setting an access restriction here affects "
                    u"content on this and lower levels.")
    ignoreRequest = True
    ignoreContent = False
    dataManager = silvaforms.makeAdaptiveDataManager(IAccessSecurity)
    fields = silvaforms.Fields(IAccessMinimumRoleSchema)
    fields['acquired'].mode = silvaforms.DISPLAY
    actions = silvaforms.Actions(silvaforms.CancelAction())

    @silvaforms.action(
        _(u"Acquire restriction"),
        description=_(u"Set the access restriction to acquire its "
                      u"setting from the parent container."),
        available=lambda form: not form.getContent().is_acquired(),
        implements=silvaforms.IRemoverAction)
    def acquire(self):
        access = self.getContentData().getContent()
        if access.is_acquired():
            self.send_message(
                _(u"The minimum role setting was already acquired."),
                type="error")
        else:
            access.set_acquired()
            self.send_message(
                _(u"Now acquiring the minimum role setting."),
                type="feedback")
        return silvaforms.SUCCESS

    @silvaforms.action(
        _(u"Set restriction"),
        description=_(
            u"Restrict access to this content to the selected role"))
    def restrict(self):
        data, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        role = data['minimum_role']
        access = self.getContent()
        if role:
            access.set_minimum_role(role)
            self.send_message(
                _(u'The minimum required role to access this item '
                  u'has been set to "${role}".',
                  mapping=dict(role=role)),
                type=u"feedback")
        else:
            access.set_acquired()
            self.send_message(
                _(u"The minimum required role to access this item is acquired."),
                type=u"feedback")
        return silvaforms.SUCCESS



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl.security import checkPermission

from five import grok
from zope.interface import Interface
from zope import schema
from zope.traversing.browser import absoluteURL
from zope.cachedescriptors.property import Lazy

from silva.core.interfaces import IContainer, IPublication, IFolder, IRoot
from silva.core.interfaces import ISilvaObject, IGhostFolder
from silva.core.interfaces import IOrderableContainer, IOrderManager
from silva.core.interfaces.adapters import ISiteManager
from silva.core.smi.content.metadata import MetadataFormGroup
from silva.core.smi.settings import Settings
from silva.core.views import views as silvaviews
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zeam.form.silva.interfaces import IRemoverAction

from Products.Silva import mangle


class ConvertToFolderAction(silvaforms.Action):
    title = _('Convert to folder')
    description = _('Change container to a folder')
    accesskey = 'ctrl+f'

    def available(self, form):
        return (IGhostFolder.providedBy(form.context) or
                (IPublication.providedBy(form.context) and
                 not form.manager.is_site()))

    def __call__(self, form):
        form.context.to_folder()
        form.send_message(_("Changed into folder."), type="feedback")
        return silvaforms.SUCCESS


class ConvertToPublicationAction(silvaforms.Action):
    title = _('Convert to publication')
    description = _('Change container to a publication')
    accesskey = 'ctrl+p'

    def available(self, form):
        return not IPublication.providedBy(form.context)

    def __call__(self, form):
        form.context.to_publication()
        form.send_message(_("Changed into publication."), type="feedback")
        return silvaforms.SUCCESS


class MakeLocalSiteAction(silvaforms.Action):
    title = _("Make local site")

    def available(self, form):
        return (IPublication.providedBy(form.context) and
                not form.manager.is_site())

    def __call__(self, form):
        try:
            form.manager.make_site()
        except ValueError as error:
            form.send_message(str(error), type=u"error")
            return silvaforms.FAILURE
        else:
            form.send_message(_("Local site activated."), type=u"feedback")
            return silvaforms.SUCCESS


class RemoveLocalSiteAction(silvaforms.Action):
    grok.implements(IRemoverAction)
    title = _("Remove local site")

    def available(self, form):
        return (IPublication.providedBy(form.context) and
                form.manager.is_site())

    def __call__(self, form):
        try:
            form.manager.delete_site()
        except ValueError as error:
            form.send_message(str(error), type=u"error")
            return silvaforms.FAILURE
        else:
            form.send_message(_("Local site deactivated."), type=u"feedback")
            return silvaforms.SUCCESS


class ContainerTypeForm(silvaforms.SMISubForm):
    grok.context(IFolder)
    grok.require('silva.ManageSilvaContent')
    grok.view(Settings)
    grok.order(10)
    actions = silvaforms.Actions(
        ConvertToPublicationAction(),
        ConvertToFolderAction(),
        MakeLocalSiteAction(),
        RemoveLocalSiteAction())
    label = _('Container type')

    def available(self):
        if IRoot.providedBy(self.context):
            return False
        if not checkPermission('silva.ManageSilvaContent', self.context):
            return False
        return super(ContainerTypeForm, self).available()

    @Lazy
    def manager(self):
        return ISiteManager(self.context)

    @Lazy
    def description(self):
        if IGhostFolder.providedBy(self.context):
            return _(u'This Ghost Folder can be converted '
                     u'to a normal Publication or Folder. All ghosted content '
                     u'will be duplicated and can then be edited.')
        elif IPublication.providedBy(self.context):
            if self.manager.is_site():
                return _(u"This Silva Publication is a local site. You need to "
                         u"remove any local service, and the local site before "
                         u"you can convert it to a Silva Folder.")
            return _(u'This Silva Publication can be converted '
                     u'to a Silva Folder, or can become a local site.')
        return _(u'This Silva Folder can be converted '
                 u'to a Publication.')


class RepairContainerOrderAction(silvaforms.Action):
    title = _('Repair order')

    def __call__(self, form):
        manager = IOrderManager(form.context, None)
        if manager is not None:
            if manager.repair(form.context.objectValues()):
                form.send_message(_("Container order was repaired."),
                                  type=u"feedback")
                return silvaforms.SUCCESS
        form.send_message(_("Container order is consistent."), type=u"feedback")
        return silvaforms.SUCCESS


class ContainerManagementForm(silvaforms.SMISubForm):
    grok.context(IOrderableContainer)
    grok.require('zope2.ViewManagementScreens')
    grok.view(Settings)
    grok.order(15)

    label = _('Container management')
    actions = silvaforms.Actions(
        RepairContainerOrderAction())

    def available(self):
        if not checkPermission('zope2.ViewManagementScreens', self.context):
            return False
        return True


class IActivateFeedsSchema(Interface):
    allow = schema.Bool(
        title=_('Allow feeds'),
        description=_('Check to provide an Atom / RSS '
                      'feed from this container.'))


class FeedsForm(silvaforms.SMISubForm):
    grok.context(IContainer)
    grok.view(Settings)
    grok.order(30)
    grok.require('silva.ManageSilvaContent')

    fields = silvaforms.Fields(IActivateFeedsSchema)
    fields['allow'].defaultValue = lambda f: bool(f.context.allow_feeds())

    ignoreContent = True
    ignoreRequest = False

    label = _('Atom/rss feeds')

    def available(self):
        if not checkPermission('silva.ManageSilvaContent', self.context):
            return False
        return super(FeedsForm, self).available()

    @silvaforms.action(_('Change feed settings'),
        identifier='activatefeeds',
        description=_('change feed settings'))
    def activate_feeds(self):
        data, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        self.context.set_allow_feeds(data.getWithDefault('allow'))
        self.send_message(_('Feed settings saved.'), type='feedback')
        return silvaforms.SUCCESS


class FeedsInformation(silvaviews.Viewlet):
    grok.context(IContainer)
    grok.order(10)
    grok.view(Settings)
    grok.viewletmanager(silvaforms.SMIFormPortlets)

    def update(self):
        self.activated = self.context.allow_feeds()
        self.url = absoluteURL(self.context, self.request)



class IQuotaSchema(Interface):
    size = schema.TextLine(title=_('Space used in this folder'))


def get_used_space(form):
    return mangle.Bytes(form.context.used_space)


class QuotaForm(silvaforms.SMISubForm):
    grok.context(IContainer)
    grok.view(Settings)
    grok.order(40)
    grok.require('silva.ManageSilvaContentSettings')

    fields = silvaforms.Fields(IQuotaSchema)
    fields['size'].defaultValue = get_used_space
    fields['size'].mode = silvaforms.DISPLAY

    # read-only
    ignoreContent = True
    ignoreRequest = True

    label = _('Used space')

    def available(self):
        return bool(
            self.context.service_extensions.get_quota_subsystem_status())

    def update(self):
        if self.available():
            self.description = \
                _('The quota for this area is set to ${quota} MB.',
                mapping={'quota': self.context.get_current_quota()})


class SettingsMetadataForm(MetadataFormGroup):
    grok.context(ISilvaObject)
    grok.order(50)
    grok.view(Settings)
    category = 'settings'
    label = _('Generic Settings')





# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from silva.core.interfaces import IContainer, IRoot, IAddableContents
from silva.translations import translate as _
from silva.ui.menu import MenuItem
from silva.core.smi.settings import SettingsMenu, Settings
from zeam.form import silva as silvaforms
from zope import schema
from zope.interface import Interface
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


@grok.provider(IContextSourceBinder)
def addables_content_types(context):
    addables = IAddableContents(context).get_all_addables()
    entries = []
    for addable in addables:
        entries.append(SimpleTerm(value=addable,
                                  token=addable,
                                  title=addable))
    return SimpleVocabulary(entries)

def default_content_types(form):
    return IAddableContents(form.context).get_container_addables()

def acquire_addables(form):
    return form.context.is_silva_addables_acquired()


class IAddablesSchema(Interface):
    acquire = schema.Bool(
        title=_(u"Acquire"),
        description=_(u"If checked, inherit the addables from above. "
                      u"Uncheck this to change the allowed addables below."),
        required=False,
        default=True)
    addables = schema.Set(
        title=_(u"Addables"),
        description=_(u"If 'acquire' is not checked, these settings will "
                      u"determine the items allowed in this publication. "
                      u"Otherwise the addables are acquired from the "
                      u"publication above."),
        required=False,
        value_type=schema.Choice(source=addables_content_types))


class AddablesForm(silvaforms.SMIForm):
    """Control addables for silva containers.
    """
    grok.adapts(Settings, IContainer)
    grok.name('addables')
    grok.require('silva.ManageSilvaContentSettings')

    label = _(u"Addable settings")
    fields = silvaforms.Fields(IAddablesSchema)
    fields['acquire'].defaultValue = acquire_addables
    fields['addables'].defaultValue = default_content_types
    actions = silvaforms.Actions(silvaforms.CancelAction())
    ignoreContent=True
    ignoreRequest=True

    def update(self):
        if IRoot.providedBy(self.context):
            self.fields['acquire'].mode = silvaforms.DISPLAY
        else:
            self.fields['acquire'].mode = silvaforms.INPUT

    @silvaforms.action(
        _(u'Save addables settings'),
        description=_(u'save addable settings for this publication'),
        implements=silvaforms.IDefaultAction,
        accesskey='ctrl+s')
    def save_changes(self):
        data, errors = self.extractData()
        currently_acquired = self.context.is_silva_addables_acquired()
        will_be_acquired = data.getWithDefault('acquire')
        if will_be_acquired and currently_acquired:
            self.send_message(
                _(u"Addable settings have not changed and remain acquired."),
                type=u"error")
            return silvaforms.FAILURE

        if will_be_acquired:
            self.send_message(
                _(u"Addable settings are now aquired."), type=u"feedback")
            self.context.set_silva_addables_allowed_in_container(None)
        else:
            self.send_message(
                _(u"Changes to addables content types saved."),
                type=u"feedback")
            self.context.set_silva_addables_allowed_in_container(
                data.getWithDefault('addables'))

        return silvaforms.SUCCESS


class AddablesMenu(MenuItem):
    grok.adapts(SettingsMenu, IContainer)
    grok.order(50)
    grok.require('silva.ManageSilvaContentSettings')
    name = _(u'Addables')
    screen = AddablesForm




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest
from zope.component import getUtility
from Products.Silva.ftesting import rest_settings
from Products.Silva.testing import FunctionalLayer, Transaction
from silva.core.references.interfaces import IReferenceService
from silva.core.references.reference import BrokenReferenceError
from silva.core.references.reference import get_content_id


class BreakReferenceTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'editor'

    def setUp(self):
        self.root = self.layer.get_application()
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFile('file', 'File')
            factory.manage_addLink('data', 'Link to File', target=self.root.file)

    def test_delete_referenced_content(self):
        with self.layer.get_browser(rest_settings) as browser:
            browser.options.handle_errors = False
            browser.login(self.user)
            with self.assertRaises(BrokenReferenceError):
                browser.open(
                    '/root/++rest++silva.ui.listing.delete',
                    method="POST",
                    form={'content': get_content_id(self.root.file)})

    def test_break_references(self):
        get_references_to = getUtility(IReferenceService).get_references_to

        self.assertNotEqual(list(get_references_to(self.root.file)), [])
        self.assertIsNot(self.root._getOb('file'), None)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.delete',
                    method="POST",
                    form={'content': get_content_id(self.root.file)}),
                400)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertEqual(
                browser.open(
                    '/root/file/++rest++silva.core.smi.breakreferences',
                    method="POST",
                    form={'form.action.break-references': 'Break'}),
                401)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertNotEqual(list(get_references_to(self.root.file)), [])
            self.assertIsNot(self.root._getOb('file'), None)


class ManagerBreakReferenceTestCase(BreakReferenceTestCase):
    user = 'manager'

    def test_break_references(self):
        get_references_to = getUtility(IReferenceService).get_references_to

        self.assertNotEqual(list(get_references_to(self.root.file)), [])
        self.assertIsNot(self.root._getOb('file'), None)
        with self.layer.get_browser(rest_settings) as browser:
            browser.login(self.user)
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.delete',
                    method="POST",
                    form={'content': get_content_id(self.root.file)}),
                400)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertEqual(
                browser.open(
                    '/root/file/++rest++silva.core.smi.breakreferences',
                    method="POST",
                    form={'form.action.break-references': 'Break'}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertItemsEqual(list(get_references_to(self.root.file)), [])
            self.assertIsNot(self.root._getOb('file'), None)
            # To debug this issue
            browser.options.handle_errors = False
            self.assertEqual(
                browser.open(
                    '/root/++rest++silva.ui.listing.delete',
                    method="POST",
                    form={'content': get_content_id(self.root.file)}),
                200)
            self.assertEqual(
                browser.content_type,
                'application/json')
            self.assertIs(self.root._getOb('file', None), None)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(BreakReferenceTestCase))
    suite.addTest(unittest.makeSuite(ManagerBreakReferenceTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope import schema
from zope.interface import Interface
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from silva.ui.menu import MenuItem
from silva.core.smi.settings import SettingsMenu, Settings
from silva.core.interfaces import ISilvaObject
from silva.core.layout.interfaces import IMarkManager
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zeam.form.silva.interfaces import IRemoverAction


class ManageCustomizeMarker(silvaforms.SMIComposedForm):
    """This form lets you add and remove customization markers for the
    current content.
    """
    grok.adapts(Settings, ISilvaObject)
    grok.name('customization')
    grok.require('silva.ManageSilvaContent')

    label = _(u"Customization markers")
    description = _(u"This screen lets you tag items with markers "
                    u"which can alter how content is displayed and behaves.")


class CustomizationMenu(MenuItem):
    grok.adapts(SettingsMenu, ISilvaObject)
    grok.order(100)
    grok.require('silva.ManageSilvaContent')
    name = _(u'Customization')
    screen = ManageCustomizeMarker


# Forms to mark objects


def interfaceTerm(interface):
    """Create a vocabulary term to represent an interface.
    """
    title = interface.__doc__
    if '\n' in title:
        title = title.split('\n', 1)[0].strip()
    if not title:
        title = interface.__identifier__
    return SimpleTerm(token=interface.__identifier__,
                      value=interface,
                      title=title)


@grok.provider(IContextSourceBinder)
def usedInterfacesForContent(context):
    manager = IMarkManager(context)
    return SimpleVocabulary(
        [interfaceTerm(interface)
         for interface in manager.usedInterfaces])


@grok.provider(IContextSourceBinder)
def usedMarkersForContent(context):
    manager = IMarkManager(context)
    return SimpleVocabulary(
        [interfaceTerm(interface)
         for interface in manager.usedMarkers])


@grok.provider(IContextSourceBinder)
def availableMarkersForContent(context):
    manager = IMarkManager(context)
    return SimpleVocabulary(
        [interfaceTerm(interface)
         for interface in manager.availableMarkers])


class IDisplayUsedInterfaces(Interface):

    usedInterface = schema.Set(
        title=_(u"Used interfaces"),
        value_type=schema.Choice(
            source=usedInterfacesForContent),
        required=False)


class IRemoveCustomizationMarker(Interface):

    usedMarkers = schema.Set(
        title=_(u"Used markers"),
        value_type=schema.Choice(
            source=usedMarkersForContent))


class IAddCustomizationMarker(Interface):

    availablesMarkers = schema.Set(
        title=_(u"Available markers"),
        value_type=schema.Choice(
            source=availableMarkersForContent))


class ContentInterfaces(grok.Adapter):
    grok.context(ISilvaObject)
    grok.provides(IDisplayUsedInterfaces)

    @property
    def usedInterface(self):
        manager = IMarkManager(self.context)
        return manager.usedInterfaces


class DisplayUsedInterfaces(silvaforms.SMISubForm):
     grok.view(ManageCustomizeMarker)
     grok.order(10)
     grok.context(ISilvaObject)

     label = _(u"Interfaces in use which affect the behavior of the item")
     fields = silvaforms.Fields(IDisplayUsedInterfaces)
     mode = silvaforms.DISPLAY
     dataManager = silvaforms.makeAdaptiveDataManager(IDisplayUsedInterfaces)
     ignoreContent = False
     ignoreRequest = True


class AddCustomizationMarker(silvaforms.SMISubForm):
    grok.view(ManageCustomizeMarker)
    grok.order(20)
    grok.context(ISilvaObject)

    label = _(u"Add a marker to alter the behavior")
    fields = silvaforms.Fields(IAddCustomizationMarker)

    def available(self):
        markers = self.fields['availablesMarkers'].valueField
        return len(markers.getChoices(self))

    @silvaforms.action(
        _(u"Add marker"),
        description=_(u"mark the item with the selected marker(s)"),
        accesskey='ctrl+a')
    def add(self):
        values, errors = self.extractData()
        if not values.get('availablesMarkers', None):
            self.send_message(_(u"You need to select a marker."), type=u"error")
            return silvaforms.FAILURE

        manager = IMarkManager(self.context)
        for value in values['availablesMarkers']:
            manager.add_marker(value)
        self.send_message(_(u"Marker added."), type=u"feedback")
        return silvaforms.SUCCESS


class RemoveCustomizationMarker(silvaforms.SMISubForm):
    grok.view(ManageCustomizeMarker)
    grok.order(30)
    grok.context(ISilvaObject)

    label = _(u"Remove a marker")
    fields = silvaforms.Fields(IRemoveCustomizationMarker)

    def available(self):
        markers = self.fields['usedMarkers'].valueField
        return len(markers.getChoices(self))

    @silvaforms.action(
        _(u"Remove"),
        description=_(u"remove the selected marker(s) from the item"),
        accesskey='ctrl+r',
        implements=IRemoverAction)
    def remove(self):
        values, errors = self.extractData()
        if not values.get('usedMarkers', None):
            self.send_message(_(u"You need to select a marker."), type=u"error")
            return silvaforms.FAILURE

        manager = IMarkManager(self.context)
        for value in values['usedMarkers']:
            manager.remove_marker(value)
        self.send_message(_(u"Marker removed."), type=u"feedback")
        return silvaforms.SUCCESS





# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import FunctionalLayer
from Products.Silva.ftesting import smi_settings


def smi_set_quota(browser, quota, should_fail=False):
    """Set the quota value, and expect a reply or an error.
    """

    form = browser.get_form('form')
    form.get_control('silva-quota.quota:record').value = quota
    assert form.get_control('save_metadata:method').click() == 200
    if should_fail:
        assert browser.inspect.form_error == [
            "The quota can't be negative or bigger than " \
                "the quota of the parent container."]
    else:
        assert browser.inspect.form_error == []
        assert browser.inspect.feedback == ['Metadata saved.']


def quota_settings(browser):
    smi_settings(browser)
    browser.inspect.add(
        'used_space',
        xpath='//div[@data-field-prefix="form.quotaform.field.size"]/div[@class="form-field"]')
    browser.inspect.add(
        'quota_space',
        xpath='//div[@data-field-prefix="form.quotaform.field.quota"]/div[@class="form-field"]')
    browser.macros.add('set_quota', smi_set_quota)


class EnableQuotaFunctionalTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')

    def test_enable_quota(self):
        """Go in ZMI, and on service_extension enable the quota system.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('manager')
        browser.open('/root/manage_main')

        self.assertTrue('service' in browser.inspect.zmi_tabs)
        browser.inspect.zmi_tabs['service'].click()
        self.assertEqual(browser.location, '/root/manage_services')
        browser.inspect.zmi_listing['service_extensions'].click()
        self.assertEqual(
            browser.inspect.zmi_title,
            ['Extensions management'])

        form = browser.get_form('general')
        form.get_control('enable_quota_subsystem').click()
        self.assertEqual(
            browser.inspect.zmi_feedback,
            ['Quota sub-system enabled'])

class QuotaFunctionalTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        # Enable the quota system
        self.root.service_extensions.enable_quota_subsystem()
        factory = self.root.manage_addProduct['Silva']
        # Create test content
        factory.manage_addPublication('publication', 'Publication')

    def test_disable_quota(self):
        """Disable the quota system.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('manager')
        browser.open('/root/manage_main')

        self.assertTrue('service' in browser.inspect.zmi_tabs)
        browser.inspect.zmi_tabs['service'].click()
        browser.inspect.zmi_listing['service_extensions'].click()
        self.assertEqual(
            browser.inspect.zmi_title,
            ['Extensions management'])

        form = browser.get_form('general')
        form.get_control('disable_quota_subsystem').click()
        self.assertEqual(
            browser.inspect.zmi_feedback,
            ['Quota sub-system disabled'])

    def test_add_asset_overquota(self):
        """Test adding an asset (file and image) being overquota.
        """
        browser = self.layer.get_web_browser(quota_settings)
        browser.login('manager')
        browser.open('/root/edit')

        # Set quota to 1MB
        browser.macros.set_quota(1)
        self.assertEqual(browser.inspect.used_space, ['0'])
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 1 MB.)'])

        # Add a file and check the used space changed
        self.assertEqual(browser.inspect.tabs['contents'].name.click(), 200)

        form = browser.get_form('md.container')
        form.controls['md.container.field.content'].value = 'Silva File'
        self.assertEqual(form.controls['md.container.action.new'].click(), 200)

        odt_filename = test_filename('docs_export_2008-06-11.odt')
        form = browser.get_form('addform')
        form.get_control('addform.field.file').value = odt_filename
        self.assertEqual(form.get_control('addform.action.save').click(), 200)

        # Add you have an error.
        self.assertEqual(
            browser.html.xpath(
                'normalize-space(//body/div[contains(@class,"simple-page")]/h1/text())'),
            "Over Quota")
        self.assertEqual(
            browser.html.xpath(
                'normalize-space(//body/div[contains(@class,"simple-page")]/p[1]/text())'),
            "A quota is applied on the container you are working on, "
            "and the current operation would require 387.1 k more space.")

        self.assertEqual(
            browser.get_link('click here to continue').click(),
            200)
        self.assertEqual(browser.location, '/root/edit')
        self.assertFalse('odt' in browser.inspect.folder_listing)

    def test_add_asset(self):
        """Test adding an asset while the quota is on.
        """
        browser = self.layer.get_web_browser(quota_settings)
        browser.login('manager')

        self.assertEqual(browser.open('/root/edit'), 200)

        # Set quota to 10MB
        browser.macros.set_quota(10)
        self.assertEqual(browser.inspect.used_space, ['0'])
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])

        # Add a file and check the used space changed
        self.assertEqual(browser.inspect.tabs['contents'].name.click(), 200)
        browser.macros.create(
            'Silva File', id='odt', title='ODT Document',
            file=test_filename('docs_export_2008-06-11.odt'))
        self.assertEqual(browser.inspect.folder_listing['odt'].click(), 200)
        self.assertEqual(browser.location, '/root/odt/edit/tab_edit')

        self.assertEqual(browser.inspect.tabs['properties'].click(), 200)
        self.assertEqual(browser.inspect.subtabs['settings'].click(), 200)
        self.assertEqual(browser.location, '/root/odt/edit/tab_settings')
        self.assertEqual(browser.inspect.used_space, ['1.4 MB'])
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])

        self.assertEqual(browser.inspect.navigation['root'].click(), 200)
        self.assertEqual(browser.location, '/root/edit/tab_settings')
        self.assertEqual(browser.inspect.used_space, ['1.4 MB'])
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])

    def test_set_quota(self):
        """Test modification of the quota's value.
        """
        browser = self.layer.get_web_browser(quota_settings)
        browser.login('manager')

        browser.open('/root/edit')

        self.assertTrue('settings' in browser.inspect.content_tabs)
        browser.inspect.content_tabs['settings'].click()
        self.assertTrue('settings' in browser.inspect.content_subtabs)
        browser.inspect.content_subtabs['settings'].click()

        self.assertEqual(browser.inspect.used_space, ['0'])
        self.assertEqual(browser.inspect.quota_space, [])

        # check and set quota
        browser.macros.set_quota(10)
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])

        # go to the publication
        browser.inspect.content_tabs['contents'].click()
        self.assertEqual(browser.inspect.folder_identifier, ['publication'])
        browser.inspect.folder_goto[0].click()

        self.assertTrue('settings' in browser.inspect.content_tabs)
        browser.inspect.content_tabs['settings'].click()
        self.assertTrue('settings' in browser.inspect.content_subtabs)
        browser.inspect.content_subtabs['settings'].click()

        self.assertEqual(browser.inspect.used_space, ['0'])
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])

        # put a correct quota, which is smaller than the parent one, 10MB.
        browser.macros.set_quota(5)
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 5 MB.)'])

        # put a quota to 0, the value of the parent one is expected.
        browser.macros.set_quota(0)
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])
        browser.macros.set_quota('')
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])

        # put a quota of 100Mb which is bigger than the one of the parent
        browser.macros.set_quota(100, should_fail=True)
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])

        # put a quota which is negative
        browser.macros.set_quota(-3, should_fail=True)
        self.assertEqual(
            browser.inspect.quota_space,
            ['(The quota for this area is set to 10 MB.)'])


def test_suite():
    suite = unittest.TestSuite()
    return suite
    suite.addTest(unittest.makeSuite(EnableQuotaFunctionalTestCase))
    suite.addTest(unittest.makeSuite(QuotaFunctionalTestCase))




# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.interface import classImplements
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from silva.core import conf as silvaconf
from silva.core.interfaces import IContent, INonPublishable
from silva.translations import translate as _
from silva.ui.interfaces import IUIScreen
from silva.ui.menu import ExpendableMenuItem, ContentMenu
from zeam.form import silva as silvaforms


class IEditScreen(IUIScreen):
    """Marker used to mark edit tab.
    """

# We cannot do it with one unique class, because of Container

class EditMenu(ExpendableMenuItem):
    grok.adapts(ContentMenu, INonPublishable)
    grok.order(10)
    name = _('Edit')
    screen = 'content'
    interface = IEditScreen


class ContentEditMenu(EditMenu):
    grok.adapts(ContentMenu, IContent)


classImplements(silvaforms.SMIEditForm, IEditScreen)


class ICompareResources(IDefaultBrowserLayer):
    silvaconf.resource('compare.css')



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.component import getUtility, getMultiAdapter
from zope.intid.interfaces import IIntIds

from infrae.comethods import cofunction

from silva.core.messages.interfaces import IMessageService
from silva.core.interfaces import IContainer, IContainerManager, IOrderManager
from silva.core.interfaces import IPublicationWorkflow
from silva.core.interfaces.errors import VersioningError
from silva.ui.interfaces import IJSView
from silva.ui.rest.base import Screen, PageREST, UIREST
from silva.ui.rest.helper import get_notifications
from silva.ui.rest.container import FolderActionREST
from silva.ui.menu import ExpendableMenuItem, ContentMenu
from silva.translations import translate as _


class Container(PageREST):
    grok.adapts(Screen, IContainer)
    grok.name('content')
    grok.require('silva.ReadSilvaContent')

    def payload(self):
        view = getMultiAdapter(
            (self.context, self.request), IJSView, name='container')
        return view(self)


class ContainerMenu(ExpendableMenuItem):
    grok.adapts(ContentMenu, IContainer)
    grok.order(10)
    grok.require('silva.ReadSilvaContent')
    name = _('Content')
    screen = Container


class DeleteActionREST(FolderActionREST):
    grok.name('silva.ui.listing.delete')
    grok.require('silva.ChangeSilvaContent')

    def payload(self):
        with IContainerManager(self.context).deleter() as deleter:
            with self.notifier(
                deleter,
                u"Deleted ${contents}.",
                u"Could not delete ${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents())
        return {}


class PasteActionREST(FolderActionREST):
    grok.name('silva.ui.listing.paste')
    grok.require('silva.ChangeSilvaContent')

    def payload(self):
        manager = IContainerManager(self.context)

        with manager.copier() as copier:
            with self.notifier(
                copier,
                u"Pasted as a copy ${contents}.",
                u"Could not copy ${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents('copied'))

        with manager.mover() as mover:
            with self.notifier(
                mover,
                u"Moved ${contents}.",
                u"Could not move ${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents('cutted'))

        return {}


class PasteAsGhostActionREST(FolderActionREST):
    grok.name('silva.ui.listing.pasteasghost')
    grok.require('silva.ChangeSilvaContent')

    def payload(self):
        with IContainerManager(self.context).ghoster() as ghoster:
            with self.notifier(
                ghoster,
                u"Created ghost for ${contents}.",
                u"Could not create ghost for ${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents('copied'))
        return {}


class RenameActionREST(FolderActionREST):
    grok.name('silva.ui.listing.rename')
    grok.require('silva.ChangeSilvaContent')

    def get_renaming_information(self):
        form = self.request.form
        total = int(form.get('values', 0))

        for index in range(total):
            for content in self.get_contents(form.get('values.%d.id' % index)):
                # This will loop at best one time, none if there is a problem
                identifier = form.get('values.%d.identifier' % index)
                title = form.get('values.%d.title' % index)
                if identifier is not None or title is not None:
                    yield (content, identifier, title)

    def payload(self):
        with IContainerManager(self.context).renamer() as renamer:
            with self.notifier(
                renamer,
                u"Renamed ${contents}.",
                u"Could not rename ${contents}: ${reason}") as notifier:
                notifier.map(self.get_renaming_information())
        return {}


class PublicationFolderActionREST(FolderActionREST):
    grok.baseclass()

    def workflow_action(self, workflow):
        raise NotImplementedError

    @cofunction
    def workflow_processor(self):
        content = yield
        while content is not None:
            workflow = IPublicationWorkflow(content, None)
            if workflow is not None:
                try:
                    self.workflow_action(workflow)
                except VersioningError as error:
                    result = error
                else:
                    result = content
            else:
                result = VersioningError(
                    _(u"This action is not applicable on this item."),
                    content)
            content = yield result


class PublishActionREST(PublicationFolderActionREST):
    grok.name('silva.ui.listing.publish')

    def workflow_action(self, workflow):
        workflow.publish()

    def payload(self):
        with self.workflow_processor() as processor:
            with self.notifier(
                processor,
                u"Published ${contents}.",
                u"Could not publish ${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents(recursive=True))
        return {}


class CloseActionREST(PublicationFolderActionREST):
    grok.name('silva.ui.listing.close')

    def workflow_action(self, workflow):
        workflow.close()

    def payload(self):
        with self.workflow_processor() as processor:
            with self.notifier(
                processor,
                u"Closed ${contents}.",
                u"Could not close ${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents(recursive=True))
        return {}


class NewVersionActionREST(PublicationFolderActionREST):
    grok.name('silva.ui.listing.newversion')

    def workflow_action(self, workflow):
        workflow.new_version()

    def payload(self):
        with self.workflow_processor() as processor:
            with self.notifier(
                processor,
                u"New version(s) created for ${contents}.",
                u"Could not create new version(s) for "
                u"${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents())
        return {}


class RejectRequestActionREST(PublicationFolderActionREST):
    grok.name('silva.ui.listing.rejectrequest')

    def workflow_action(self, workflow):
        workflow.reject_request()

    def payload(self):
        with self.workflow_processor() as processor:
            with self.notifier(
                processor,
                u"Publication rejected for ${contents}.",
                u"Could not reject publication for "
                u" ${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents())
        return {}


class WithdrawRequestActionREST(PublicationFolderActionREST):
    grok.name('silva.ui.listing.withdrawrequest')

    def workflow_action(self, workflow):
        workflow.withdraw_request()

    def payload(self):
        with self.workflow_processor() as processor:
            with self.notifier(
                processor,
                u"Publication request withdrawn for ${contents}.",
                u"Could not withdraw publication request for "
                u"${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents())
        return {}


class RequestApprovalActionREST(PublicationFolderActionREST):
    grok.name('silva.ui.listing.requestapproval')

    def workflow_action(self, workflow):
        workflow.request_approval()

    def payload(self):
        with self.workflow_processor() as processor:
            with self.notifier(
                processor,
                u"Approval for publication requested for ${contents}.",
                u"Could not request publication approval for "
                u"${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents())
        return {}


class RevokeApprovalActionREST(PublicationFolderActionREST):
    grok.name('silva.ui.listing.revokeapproval')

    def workflow_action(self, workflow):
        workflow.revoke_approval()

    def payload(self):
        with self.workflow_processor() as processor:
            with self.notifier(
                processor,
                u"Approval canceled for ${contents}.",
                u"Could not cancel approval for "
                u"${contents}: ${reason}") as notifier:
                notifier.map(self.get_selected_contents())
        return {}


class OrderREST(UIREST):
    grok.context(IContainer)
    grok.name('silva.ui.listing.order')
    grok.require('silva.ChangeSilvaContent')

    def get_integer(self, name):
        try:
            return int(self.request.form.get(name, '-1'))
        except ValueError:
            return -1

    def notify(self, message, type=u""):
        service = getUtility(IMessageService)
        service.send(message, self.request, namespace=type)

    def POST(self):
        position = self.get_integer('position')
        content = self.get_integer('content')

        if (position >= 0 and content >= 0 and
            IOrderManager(self.context).move(
                getUtility(IIntIds).getObject(content),
                position)):
            success = True
            self.notify(
                _(u'Item moved in position ${position}.',
                  mapping={'position': position + 1}),
                type='feedback')
        else:
            success = False
            self.notify(
                _(u'Could not move item in position ${position}.',
                  mapping={'position': position + 1}),
                type='error')

        data = {'content': success}
        get_notifications(self, data)

        return self.json_response(data)




# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

from Acquisition import aq_parent

from five import grok
from silva.core.interfaces import ISilvaObject, IRoot
from silva.core.interfaces import IIconResolver, ISiteManager
from silva.core.interfaces import ISilvaConfigurableService
from silva.translations import translate as _
from silva.ui import rest
from silva.ui.menu import MenuItem

from .userinfo import UserSettingsMenu


class ConfigurationScreen(rest.PageWithTemplateREST):
    grok.adapts(rest.Screen, ISilvaObject)
    grok.name('admin')
    grok.require('zope2.ViewManagementScreens')
    form = False

    def get_menu_title(self):
        return _("Site Configuration")

    def get_services(self, container):
        services = []
        for candidate in container.objectValues():
            if ISilvaConfigurableService.providedBy(candidate):
                # We assume service icons are URLs
                services.append({
                        'name': candidate.meta_type,
                        'icon': self.get_icon(candidate),
                        'path': self.get_content_path(candidate)})
        return sorted(services, key=lambda i: i['name'].lower())

    def update(self):
        self.get_icon = IIconResolver(self.request).get_tag
        self.main_services = self.get_services(self.context.get_root())
        self.local_services = None
        container = self.context.get_publication()
        while not IRoot.providedBy(container):
            if ISiteManager(container).is_site():
                self.local_services = self.get_services(container)
                break
            container = aq_parent(container).get_publication()


class ConfiguationMenu(MenuItem):
    grok.adapts(UserSettingsMenu, ISilvaObject)
    grok.order(10)
    grok.require('zope2.ViewManagementScreens')
    name = _('Site Preferences')
    icon = 'configuration'
    screen = ConfigurationScreen
